import logging

log = logging.getLogger(__name__)


class Billing(object):

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_bucket_billing_info(self, bucket_name, namespace, sizeunit='GB'):
        """
        Gets billing details for the specified namespace and bucket name.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'total_objects': 0,
            u'total_size_in_gb': 0.0,
            u'name': u'bucket-test',
            u'sample_time': u'2015-06-18T22: 20: 01Z',
            u'namespace': u'namespace1',
            u'vpool_id': u'urn: storageos: ReplicationGroupInfo:
                            4ea1fa1e-a7d1-4a8e-b8cc-e5a2c27f308d: global',
            u'total_size_unit': u'GB'
        }

        :param bucket_name: Bucket name for which billing information needs
        to be retrieved
        :param namespace: Namespace containing the bucket
        :param sizeunit: Unit to be used for calculating the size on disk (KB,MB and GB. GB is default value)
        """
        log.info("Getting billing info for bucket '{0}'".format(bucket_name))

        params = {
            "sizeunit": sizeunit
        }

        return self.conn.get(
            url='object/billing/buckets/{0}/{1}/info'.format(
                namespace, bucket_name), params=params)

    def get_namespace_billing_info(self, namespace, sizeunit='GB',
                                   include_bucket_detail=False, marker=None):
        """
        Gets billing details for the specified namespace and bucket details.
        Note: Due to the fact that sampling a namespace's buckets takes some
        time it's possible that the values for the buckets will not sum to
        equal the namespace's values.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {{
            u'total_objects': 90,
            u'total_size_in_gb': 10.0,
            u'sample_time': u'2015-06-18T22: 28: 59Z',
            u'namespace': u'namespace1',
            u'bucket_billing_info': [
                {
                    u'vpool_id': u'urn: storageos: ReplicationGroupInfo:
                                4ea1fa1e-a7d1-4a8e-b8cc-e5a2c27f308d: global',
                    u'total_size_unit': u'GB',
                    u'total_objects': 0,
                    u'total_size_in_gb': 0.0,
                    u'name': u'bucket-test1'
                },
                {
                    u'vpool_id': u'urn: storageos: ReplicationGroupInfo:
                                4ea1fa1e-a7d1-4a8e-b8cc-e5a2c27f308d: global',
                    u'total_size_unit': u'GB',
                    u'total_objects': 90,
                    u'total_size_in_gb': 2.0,
                    u'name': u'large-uploads'
                }
            ],
            u'total_size_unit': u'GB',
            u'next_marker': u'test-302599c4-5ewd-44f3-b0dc-716ae3782dbd'
        }

        :param namespace: Namespace to get information about
        :param include_bucket_detail: Optional. (default=False). If True,
        include information about all the buckets owned by this namespace.
        :param marker: Optional. Used to continue a truncated response. Omit
        this parameter on the first request.
        :param sizeunit: Unit to be used for calculating the size on disk (KB,MB and GB. GB is default value)
        """
        log.info("Getting billing info for namespace '{0}'".format(namespace))

        params = {
            "include_bucket_detail": include_bucket_detail,
            "sizeunit": sizeunit
        }

        if marker:
            params['marker'] = marker

        return self.conn.get(
            url='object/billing/namespace/{0}/info'.format(
                namespace), params=params)

    def get_namespace_billing_sample(self, namespace, start_time, end_time, sizeunit='GB',
                                     include_bucket_detail=False, marker=None):
        """
        Gets billing details for the specified namespace, interval and bucket
        details. This method will return one and only one time sample. If the
        start_time and end_time do not align with a sampled time period, the
        matching samples will be aggregated.

        Check the response for the actual time period sampled. Due to the fact
        that sampling a namespace's buckets takes some time it is possible
        that the values for the buckets will not sum to equal the
        namespace's values.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'total_objects': 4707,
            u'total_size_in_gb': 10.0,
            u'bytes_delta': 0,
            u'bucket_billing_sample': [
                {
                    u'total_objects': 0,
                    u'total_size_in_gb': 0.0,
                    u'name': u'bucket-test1',
                    u'namespace': u'namespace1',
                    u'ingress': 0,
                    u'vpool_id': u'urn: storageos: ReplicationGroupInfo:
                                4ea1fa1e-a7d1-4a8e-b8cc-e5a2c27f308d: global',
                    u'objects_created': 0,
                    u'egress': 0,
                    u'sample_start_time': u'2015-06-15T00: 00: 00Z',
                    u'objects_deleted': 0,
                    u'sample_end_time': u'2015-06-15T01: 00: 00Z',
                    u'total_size_unit': u'GB',
                    u'bytes_delta': 0
                },
                {
                    u'total_objects': 0,
                    u'total_size_in_gb': 0.0,
                    u'name': u'large-uploads',
                    u'namespace': u'namespace1',
                    u'ingress': 0,
                    u'vpool_id': u'urn: storageos: ReplicationGroupInfo:
                                4ea1fa1e-a7d1-4a8e-b8cc-e5a2c27f308d: global',
                    u'objects_created': 0,
                    u'egress': 0,
                    u'sample_start_time': u'2015-06-15T00: 00: 00Z',
                    u'objects_deleted': 0,
                    u'sample_end_time': u'2015-06-15T01: 00: 00Z',
                    u'total_size_unit': u'GB',
                    u'bytes_delta': 0
                }
            ],
            u'namespace': u'namespace1',
            u'ingress': 0,
            u'objects_created': 0,
            u'egress': 0,
            u'sample_start_time': u'2015-06-15T00: 00: 00Z',
            u'objects_deleted': 0,
            u'sample_end_time': u'2015-06-15T01: 00: 00Z',
            u'total_size_unit': u'GB',
            u'next_marker': u'test-302599c4-5ewd-44f3-b0dc-716ae3782dbd'
        }

        :param namespace: Namespace to get information about
        :param include_bucket_detail: Optional. (default=False). If True,
        include information about all the buckets owned by this namespace.
        :param marker: Optional. Used to continue a truncated response. Omit
        this parameter on the first request.
        :param start_time: Starting time in ISO-8601 minute format
        :param end_time: Ending time in ISO-8601 minute format
        :param sizeunit: Unit to be used for calculating the size on disk (KB,MB and GB. GB is default value)
        """
        log.info("Sampling billing info for namespace '{0}'".format(namespace))

        params = {
            "include_bucket_detail": include_bucket_detail,
            "start_time": start_time,
            "end_time": end_time,
            "sizeunit": sizeunit
        }

        if marker:
            params['marker'] = marker

        return self.conn.get(
            url='object/billing/namespace/{0}/sample'.format(
                namespace), params=params)

    def get_bucket_billing_sample(self, bucket_name, namespace,
                                  start_time, end_time, sizeunit='GB'):
        """
        Gets billing details for the specified namespace, interval and bucket
        details. By default, buckets are sampled every 5 minutes. If the
        requested time range includes multiple samples, the data will be
        aggregated.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'total_objects': 0,
            u'total_size_in_gb': 0.0,
            u'name': u'bucket-test',
            u'namespace': u'namespace1',
            u'ingress': 0,
            u'vpool_id': u'urn: storageos: ReplicationGroupInfo:
                            4ea1fa1e-a7d1-4a8e-b8cc-e5a2c27f308d: global',
            u'objects_created': 0,
            u'egress': 0,
            u'sample_start_time': u'2015-06-15T00: 00: 00Z',
            u'objects_deleted': 0,
            u'sample_end_time': u'2015-06-15T01: 00: 00Z',
            u'total_size_unit': u'GB',
            u'bytes_delta': 0
        }

        :param namespace: Namespace containing the bucket
        :param bucket_name: Bucket name
        :param start_time: Starting time in ISO-8601 minute format
        :param end_time: Ending time in ISO-8601 minute format
        :param sizeunit: Unit to be used for calculating the size on disk (KB,MB and GB. GB is default value)s
        """
        log.info("Sampling billing info for bucket '{0}' in namespace "
                 "'{1}'".format(bucket_name, namespace))

        params = {
            "start_time": start_time,
            "end_time": end_time,
            "sizeunit": sizeunit
        }

        return self.conn.get(
            url='object/billing/buckets/{0}/{1}/sample'.format(
                namespace, bucket_name), params=params)



import logging

log = logging.getLogger(__name__)


class Authentication(object):
    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def logout(self, force=False):
        """
        Perform an HTTP GET against the ECS endpoint to
        log an authenticated user out, thereby invalidating their
        authentication token.

        Example JSON result from the API:

        {"user": "root"}

        :param force: If you have multiple sessions running simultaneously this
        forces the termination of all tokens to the current user
        """

        if not self.conn.get_current_token():
            log.warning('Not logging out since the client has no token set up')
            return

        params = {
            'force': force
        }

        log.info('Terminating session (signing out): {0}'.format(params))

        if force:
            logout_resp = self.conn.get('logout', params=params)
        else:
            logout_resp = self.conn.get('logout')

        # Remove cached authorization token from disk, as the session is
        # now terminated.
        self.conn.remove_cached_token()

        return logout_resp



from ecsclient import schemas
from tests import functional


class TestWhoami(functional.BaseTestCase):
    def test_whoami(self):
        response = self.client.user_info.whoami()
        self.assertValidSchema(response, schemas.WHOAMI)



import logging

log = logging.getLogger(__name__)


class StoragePool(object):
    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get(self, storage_pool_id):
        """
        Gets the details for the specified storage pool.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            'isProtected': False,
            'isColdStorageEnabled': False,
            'id': 'urn: storageos: VirtualArray:
                    3c4e8cca-2e3d-4f8d-b183-1c69ce2d5b37',
            'name': 'storagepool1'
        }

        :param storage_pool_id: Storage pool identifier to be retrieved
        """
        log.info("Getting storage pool '{}'".format(storage_pool_id))
        return self.conn.get('vdc/data-services/varrays/{}'.format(storage_pool_id))

    def list(self, vdc_id=None):
        """
        Gets a list of storage pools from the local VDC.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            'varray': [
                {
                    'isProtected': False,
                    'isColdStorageEnabled': False,
                    'id': 'urn: storageos: VirtualArray:
                            3c4e8cca-2e3d-4f8d-b183-1c69ce2d5b37',
                    'name': 'storagepool1'
                },
                {
                    'isProtected': True,
                    'isColdStorageEnabled': False,
                    'id': 'urn: storageos: VirtualArray:
                            c7fc54dc-6616-4b7e-a86c-0210ef9a8804',
                    'name': 'storagepool2'
                }
            ]
        }

        :param vdc_id: Virtual data center identifier for which list of storage pool is to be retrieved
        """
        msg = "Listing storage pools"
        params = None
        if vdc_id:
            params = {'vdc-id': vdc_id}
            msg += " for VDC '{}'".format(vdc_id)
        log.info(msg)
        return self.conn.get('vdc/data-services/varrays', params=params)

    def create(self, name, description=None, is_protected=False, is_cold_storage_enabled=False):
        """
        Create a storage pool with the specified details.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        {
            'id': 'urn:storageos:VirtualArray:dd751e72-142-598-b4f833e93b61',
            'name': 'storage_pool',
            'isProtected': False,
            'isColdStorageEnabled': False
        }

        :param name: Storage pool name
        :param description: Storage pool description
        :param is_protected: Set to True to enable storage pool protection, False otherwise
        :param is_cold_storage_enabled: Set to True to enable cold storage, False otherwise
        """
        if not description:
            description = name

        payload = {
            "name": name,
            "description": description,
            "isProtected": is_protected,
            "isColdStorageEnabled": is_cold_storage_enabled
        }
        log.info("Creating storage pool '{}'".format(name))
        return self.conn.post('vdc/data-services/varrays', json_payload=payload)

    def update(self, storage_pool_id, name=None, is_protected=None):
        """
        Updates storage pool for the specified identifier.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        {
            'id': 'urn:storageos:VirtualArray:82cf257b-0782-433c-92ca-2ee6161b917e',
            'name': 'CommodityvPool',
            'description': 'desc',
            'isProtected': True,
            'isColdStorageEnabled': False
        }

        :param storage_pool_id: Storage pool identifier to be updated
        :param name: Name of virtual array to be updated
        :param is_protected: Set to True to enable storage pool protection, False otherwise
        """
        payload = {
            "name": name,
            "isProtected": is_protected
        }
        # FIXME: API throws error if 'isColdStorageEnabled' and 'description' fields are sent
        log.info("Updating storage pool ID '{}'".format(storage_pool_id))
        return self.conn.put('vdc/data-services/varrays/{}'.format(storage_pool_id),
                             json_payload=payload)

    def delete(self, storage_pool_id):
        """
        Deletes the storage pool for the specified identifier.

        Required role(s):

        SYSTEM_ADMIN

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param storage_pool_id: storage pool identifier to be deleted
        """
        log.info("Deleting storage pool ID '{}'".format(storage_pool_id))
        return self.conn.delete('vdc/data-services/varrays/{}'.format(storage_pool_id))



from ecsclient import schemas
from tests import functional


class TestLicense(functional.BaseTestCase):
    def test_get_license(self):
        response = self.client.licensing.get_license()
        self.assertValidSchema(response, schemas.LICENSE)

    def test_add_license(self):
        response = self.client.licensing.add_license(self.license_text)
        print(response)



import logging

import ecsclient.v2.client as v2_client
import ecsclient.v3.client as v3_client

_logger = logging.getLogger(__name__)

_CLIENT_VERSIONS = {'2': v2_client.Client,
                    '3': v3_client.Client}


def Client(version=None, *args, **kwargs):
    """Factory function to create a new ECS client.

    The returned client will be either a V3 or V2 client. Check the version
    using the :py:attr:`~ecsclient.v2.client.Client.version` property or
    the instance's class (with instanceof).

     :param string version: The required version of the ECS Management API.
    """

    if not version:
        msg = "Please provide the API version. Options are: '2', '3'."
        raise RuntimeError(msg)

    try:
        client_class = _CLIENT_VERSIONS[version]
    except KeyError:
        msg = "No client available for version '%s'" % version
        raise RuntimeError(msg)

    return client_class(*args, **kwargs)



import json
from six.moves import urllib

from ecsclient.common import error_codes


class ECSClientException(Exception):
    """
    This is a custom ECSClientException Exception class to better handle
    errors that ECSClientException can return in HTML format rather than JSON.
    It includes the original error message from ECS (ecs_message) and the
    HTTP status code returned (http_status_code).
    """

    def __init__(self, message, ecs_code=None, ecs_retryable=None, ecs_description='',
                 ecs_details='', http_scheme='', http_host='', http_port='',
                 http_path='', http_query='', http_status=None, http_reason='',
                 http_response_content='', http_response_headers=None):
        """
        This is a custom ECSClientException class to handle API error responses
        """
        super(ECSClientException, self).__init__(message)
        self.message = message
        self.ecs_code = ecs_code
        self.ecs_retryable = ecs_retryable
        self.ecs_description = ecs_description
        self.ecs_details = ecs_details
        self.http_scheme = http_scheme
        self.http_host = http_host
        self.http_port = http_port
        self.http_path = http_path
        self.http_query = http_query
        self.http_status = http_status
        self.http_reason = http_reason
        self.http_response_content = http_response_content
        self.http_response_headers = http_response_headers

    @classmethod
    def from_response(cls, resp, message=None):
        code = None
        retryable = None
        description = ''
        details = ''
        try:
            m = json.loads(resp.text)
            code = m.get('code', None)
            retryable = m.get('retryable', None)
            description = m.get('description', '')
            details = m.get('details', '')
        except BaseException:
            pass

        parsed_url = urllib.parse.urlparse(resp.request.url)
        message = message or details or description or error_codes.ERROR_CODES.get(code) or '%s %s' % (
            resp.status_code, resp.reason)

        return cls(message,
                   ecs_code=code,
                   ecs_retryable=retryable,
                   ecs_description=description,
                   ecs_details=details,
                   http_scheme=parsed_url.scheme,
                   http_host=parsed_url.hostname,
                   http_port=parsed_url.port,
                   http_path=parsed_url.path,
                   http_query=parsed_url.query,
                   http_status=resp.status_code,
                   http_reason=resp.reason,
                   http_response_content=resp.text[:8192] or resp.content[:8192],
                   http_response_headers=resp.headers)

    def __str__(self):
        a = "\n\nError Message: %s\n" % self.message
        if self.ecs_details not in a:
            a += 'ECS_details: %s\n' % self.ecs_details
        if self.ecs_description not in a:
            a += 'ECS_description: %s\n\n' % self.ecs_description
        b = ''
        if self.http_scheme:
            b += 'HTTP/S Request/Response Details:\n---------------------------------\nScheme: %s\n' % self.http_scheme
        if self.http_host:
            b += "Host: %s\n" % self.http_host
        if self.http_port:
            b += 'Port:%s\n' % self.http_port
        if self.http_path:
            b += 'Path: %s\n' % self.http_path
        if self.http_query:
            b += 'Query: %s\n' % self.http_query

        if self.http_status:
            if b:
                b += 'Response_Status: %s\n' % str(self.http_status)
            else:
                b = 'HTTP/S Response_Status: %s\n' % str(self.http_status)
        if self.http_reason:
            if b:
                b += 'Response_Reason: %s\n' % self.http_reason
            else:
                b = 'HTTP/S Response_Reason: %s\n' % self.http_reason
        if self.http_response_content:
            b += 'Response_content: %s' % self.http_response_content

        return b and a + b or a



import logging

log = logging.getLogger(__name__)


class Snmp(object):
    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_snmp_agent(self):
        """
        Get SNMP Agent configuration including targets.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:
        """
        # TODO: Add example JSON response
        log.info("Fetching SNMP Agent configuration")
        return self.conn.get('vdc/snmp/config')

    def update_snmp_agent(self, engine_id):
        """
        Sets SNMP agent configuration

        Required role(s):

        SYSTEM_ADMIN

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param engine_id: Fully qualified domain name or IP and port
        """
        payload = {
            "engineID": engine_id
        }
        log.info("Setting SNMP agent configuration with Engine ID '{}'".format(engine_id))
        return self.conn.put('vdc/snmp/config', json_payload=payload)

    def get_snmp_target(self, target_id):
        """
        Get SNMP target info.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:
        {}

        :param target_id: SNMP target ID
        """
        # TODO: Add example JSON response
        log.info("Getting SNMP target information with ID '{}'".format(target_id))
        return self.conn.get('vdc/snmp/config/target/{}'.format(target_id))

    def create_snmp_target(self, server, port, version, community, username, auth_protocol,
                           auth_passphrase, privacy_protocol, privacy_passphrase):
        """
        Adds SNMP Target.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:
        {}

        Expect: HTTP/1.1 200 OK

        :param server: Fully qualified domain name or IP
        :param port: SNMP port
        :param version: SNMP version ('V2' or 'V3')
        :param community: Community name (only needed when version is 'V2')
        :param username: Username for authentication
        :param auth_protocol: Authentication protocol ('MD5' or 'SHA')
        :param auth_passphrase: Authentication passphrase
        :param privacy_protocol: Encryption protocol ('DES' or 'AES')
        :param privacy_passphrase: Encryption passphrase
        """
        # TODO: Add example JSON response

        payload = {
            "server": server,
            "port": port,
            "version": version,
            "community": community,
            "user_security_model": {
                "username": username,
                "authentication": {
                    "protocol": auth_protocol,
                    "passphrase": auth_passphrase
                },
                "privacy": {
                    "protocol": privacy_protocol,
                    "passphrase": privacy_passphrase
                }
            }
        }

        log.info("Creating SNMP target")
        return self.conn.post('vdc/snmp/config/target', json_payload=payload)

    def update_snmp_target(self, target_id, server, port, version, community, username,
                           auth_protocol, auth_passphrase, privacy_protocol,
                           privacy_passphrase):
        """
        Adds SNMP Target.

        Required role(s):

        SYSTEM_ADMIN

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param server: Fully qualified domain name or IP
        :param port: SNMP port
        :param version: SNMP version ('V2' or 'V3')
        :param community: Community name (only needed when version is 'V2')
        :param username: Username for authentication
        :param auth_protocol: Authentication protocol ('MD5' or 'SHA')
        :param auth_passphrase: Authentication passphrase
        :param privacy_protocol: Encryption protocol ('DES' or 'AES')
        :param privacy_passphrase: Encryption passphrase
        """

        payload = {
            "server": server,
            "port": port,
            "version": version,
            "community": community,
            "user_security_model": {
                "username": username,
                "authentication": {
                    "protocol": auth_protocol,
                    "passphrase": auth_passphrase
                },
                "privacy": {
                    "protocol": privacy_protocol,
                    "passphrase": privacy_passphrase
                }
            }
        }

        log.info("Updating SNMP target with ID '{}'".format(target_id))
        return self.conn.put('vdc/snmp/config/target/{}'.format(target_id),
                             json_payload=payload)

    def delete_snmp_target(self, target_id):
        """
        Delete specified SNMP target.

        Required role(s):

        SYSTEM_ADMIN

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param target_id: SNMP target ID
        """
        log.info("Deleting SNMP target with ID '{}'".format(target_id))
        return self.conn.delete('vdc/snmp/config/target/{}'.format(target_id))



import logging
import testtools
from mock import mock
from requests.auth import _basic_auth_str
from requests_mock.contrib import fixture

from ecsclient.baseclient import Client
from ecsclient.common.exceptions import ECSClientException


class TestAuthentication(testtools.TestCase):
    LOGIN_URL = 'http://127.0.0.1:4443/login'
    LOGOUT_URL = 'http://127.0.0.1:4443/logout'

    def setUp(self, *args, **kwargs):
        # logging.basicConfig(level=logging.DEBUG)
        super(TestAuthentication, self).setUp(*args, **kwargs)
        self.client = Client(username='someone',
                             password='password',
                             ecs_endpoint='http://127.0.0.1:4443',
                             token_endpoint='http://127.0.0.1:4443/login')
        self.requests_mock = self.useFixture(fixture.Fixture())

    def test_get_token_valid_credentials(self):
        self.requests_mock.register_uri('GET', self.LOGIN_URL, headers={'X-SDS-AUTH-TOKEN': 'FAKE-TOKEN-123'})
        self.assertIsNone(self.client.token)

        token = self.client.get_token()

        self.assertEqual(token, 'FAKE-TOKEN-123')
        self.assertEqual(self.client._token_request.token, 'FAKE-TOKEN-123')
        self.assertEqual(self.requests_mock.last_request.method, 'GET')
        self.assertEqual(self.requests_mock.last_request.url, self.LOGIN_URL)
        self.assertEqual(self.requests_mock.last_request.headers['authorization'],
                         _basic_auth_str('someone', 'password'))

    def test_get_token_invalid_credentials(self):
        self.requests_mock.register_uri('GET', self.LOGIN_URL, status_code=401, text='body')

        with super(testtools.TestCase, self).assertRaises(ECSClientException) as error:
            self.client.get_token()

        exception = error.exception
        self.assertIsNone(self.client._token_request.token)
        self.assertEqual(exception.message, 'Invalid username or password')
        self.assertEqual(exception.http_response_content, 'body')
        self.assertEqual(exception.http_status, 401)
        self.assertEqual(self.requests_mock.last_request.method, 'GET')
        self.assertEqual(self.requests_mock.last_request.url, self.LOGIN_URL)
        self.assertEqual(self.requests_mock.last_request.headers['authorization'],
                         _basic_auth_str('someone', 'password'))

    @mock.patch('ecsclient.baseclient.os.remove')
    @mock.patch('ecsclient.baseclient.os.path.isfile')
    def test_logout(self, mock_isfile, mock_remove):
        self.client.token = 'FAKE-TOKEN-123'
        self.client._token_request.token = 'FAKE-TOKEN-123'
        self.requests_mock.register_uri('GET', self.LOGOUT_URL, text="{'user': 'someone'}")
        mock_isfile.return_value = True
        mock_remove.return_value = True

        resp = self.client.authentication.logout()

        self.assertEqual(resp, "{'user': 'someone'}")
        self.assertIsNone(self.client.token)
        self.assertIsNone(self.client._token_request.token)
        mock_isfile.assert_called_with('/tmp/ecsclient.tkn')
        mock_remove.assert_called_with('/tmp/ecsclient.tkn')
        self.assertEqual(self.requests_mock.last_request.method, 'GET')
        self.assertEqual(self.requests_mock.last_request.url, self.LOGOUT_URL)
        self.assertEqual(self.requests_mock.last_request.headers['x-sds-auth-token'], 'FAKE-TOKEN-123')

    @mock.patch('ecsclient.baseclient.os.remove')
    @mock.patch('ecsclient.baseclient.os.path.isfile')
    def test_logout_force(self, mock_isfile, mock_remove):
        self.client.token = 'FAKE-TOKEN-123'
        self.client._token_request.token = 'FAKE-TOKEN-123'
        self.requests_mock.register_uri('GET', self.LOGOUT_URL + '?force=True', text="{'user': 'someone'}")
        mock_isfile.return_value = True
        mock_remove.return_value = True

        resp = self.client.authentication.logout(force=True)

        self.assertEqual(resp, "{'user': 'someone'}")
        self.assertIsNone(self.client.token)
        self.assertIsNone(self.client._token_request.token)
        mock_isfile.assert_called_with('/tmp/ecsclient.tkn')
        mock_remove.assert_called_with('/tmp/ecsclient.tkn')
        self.assertEqual(self.requests_mock.last_request.method, 'GET')
        self.assertEqual(self.requests_mock.last_request.url, self.LOGOUT_URL + '?force=True')
        self.assertEqual(self.requests_mock.last_request.qs['force'], ['true'])
        self.assertEqual(self.requests_mock.last_request.headers['x-sds-auth-token'], 'FAKE-TOKEN-123')

    def test_logout_when_logged_out(self):
        self.client._token_request.token = 'FAKE-TOKEN-123'
        self.client._token_request.cache_token = False
        self.requests_mock.register_uri('GET', self.LOGOUT_URL, text="{'user': 'someone'}")
        self.requests_mock.register_uri('GET', 'http://127.0.0.1:4443/user/whoami')

        resp = self.client.authentication.logout()

        self.assertEqual(resp, "{'user': 'someone'}")

        resp2 = self.client.authentication.logout()

        self.assertIsNone(resp2)

    def test_logout_and_reconnect(self):
        self.client.token = 'FAKE-TOKEN-123'
        self.client._token_request.token = 'FAKE-TOKEN-123'
        self.client._token_request.cache_token = False
        self.requests_mock.register_uri('GET', self.LOGOUT_URL, text="{'user': 'someone'}")

        self.client.authentication.logout()

        self.assertIsNone(self.client.token)
        self.assertIsNone(self.client._token_request.token)

        self.requests_mock.register_uri('GET', self.LOGIN_URL, headers={'X-SDS-AUTH-TOKEN': 'NEW-TOKEN-123'})

        self.client.get('login')

        self.assertEqual(self.client._token_request.token, 'NEW-TOKEN-123')





# Standard lib imports
import logging

# Third party imports
# None

# Project level imports
# None


log = logging.getLogger(__name__)


class TemporaryFailedZone(object):

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_all_temp_failed_zones(self):
        """
        Gets all the configured temp failed zones.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'tempfailedzone': [
                {
                    u'remote': None,
                    u'tags': [],
                    u'global': None,
                    u'rgId': u'urn: storageos: ReplicationGroupInfo:
                                c2b0d3c4-c778-4a24-8da5-6a89784c4eeb: global',
                    u'vdc': None,
                    u'inactive': False,
                    u'failedZoneList': []
                },
                {
                    u'remote': None,
                    u'tags': [],
                    u'global': None,
                    u'rgId': u'urn: storageos: ReplicationGroupInfo:
                                4ea1fa1e-a7d1-4a8e-b8cc-e5a2c27f308d: global',
                    u'vdc': None,
                    u'inactive': False,
                    u'failedZoneList': []
                }
            ]
        }
        """
        log.info("Fetching all Temporary Failed Zones")
        return self.conn.get(url='tempfailedzone/allfailedzones')

    def get_temp_failed_zone(self, replication_group_id):
        """
        Gets all the temp failed zones for the specified replication group
        identifier.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'remote': None,
            u'tags': [],
            u'global': None,
            u'rgId': u'urn: storageos: ReplicationGroupInfo:
                        c2b0d3c4-c778-4a24-8da5-6a89784c4eeb: global',
            u'vdc': None,
            u'inactive': False,
            u'failedZoneList': []
        }

        :param replication_group_id: Replication group id to retrieve details
        """
        log.info("Fetching TFZs for vpool '{0}'".format(replication_group_id))

        return self.conn.get(
            url='tempfailedzone/rgid/{0}'.format(replication_group_id))



import logging

log = logging.getLogger(__name__)


class SecretKey(object):
    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get(self, user_id=None, namespace=None):
        """
        Gets all secret keys for the specified user. If the user is not provided,
        it will use authenticated user. If the user belongs to a namespace,
        the namespace must be supplied

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'key_timestamp_2': u'2015-05-2105: 43: 33.281',
            u'key_timestamp_1': u'2015-05-2105: 43: 30.872',
            u'key_expiry_timestamp_2': u'',
            u'key_expiry_timestamp_1': u'',
            u'secret_key_1': u'3tW4A5G7QaAGVtsGLVL8uDy73S2A6LrvkPTDNdrk',
            u'secret_key_2': u'v5LjgoEM9lqHIo+qa7pIhGjrecc/Wv+WVw7kO4oQ',
            u'link': {
                u'href': u'/object/secret-keys',
                u'rel': u'self'
            }
        }

        :param user_id: Valid user identifier to get the keys from. If not provided,
        the authenticated user will be used instead.
        :param namespace: Namespace for the user if the user belongs to a namespace (optional)
        """
        if user_id:
            msg = "Getting secret keys for user '{}'".format(user_id)
            url = 'object/user-secret-keys/{}'.format(user_id)
            if namespace:
                msg += " in namespace '{}'".format(namespace)
                url += '/{}'.format(namespace)
        else:
            msg = "Getting secret keys for the authenticated user"
            url = 'object/secret-keys'

        log.info(msg)
        return self.conn.get(url)

    def create(self, user_id=None, namespace=None, expiry_time=None, secret_key=None):
        """
        Creates a secret key for the specified user. If the user is not provided,
        it will create the secret key for the authenticated user. If the user
        is provided and belongs to a namespace, the namespace must be supplied.
        You may pass in an expiration time in minutes. During the expiration interval,
        both keys will be accepted for requests. This gives you a grace period where
        you can update applications to use the new key.

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        No specific role is required if a user ID is not provided.

        Example JSON result from the API:

        {
            "link": {
                "rel": "self",
                "href":"/object/user-secret-keys/joedeleteme"
            },
            "secret_key": "p3PZyb//Ch6tM0fUsnesYYnGb+6JHV8WHzS5YHjg",
            "key_timestamp": "2014-12-24 02:08:40.181"
        }

        :param user_id: Valid user identifier to create a key for. If not provided,
        the authenticated user will be used instead.
        :param namespace: Namespace for the user if the user belongs to a namespace (optional)
        :param expiry_time: Expiry time in minutes for the previous secret key. Note that nodes may
        cache secret keys for up to two minutes so old keys may not expire immediately.
        Do not provide an expiry time if the user does not have an existing key (optional)
        :param secret_key: Secret key associated with this user. If not provided, system
        will generate one
        """

        payload = dict()

        if expiry_time:
            payload["existing_key_expiry_time_mins"] = expiry_time

        if user_id:
            url = 'object/user-secret-keys/{}'.format(user_id)
            msg = "Creating secret for user '{}'".format(user_id)
            if namespace:
                payload['namespace'] = namespace
            if secret_key:
                payload['secretkey'] = secret_key
        else:
            url = 'object/secret-keys'
            msg = "Creating secret for the authenticated user"
            if secret_key:
                log.warning("Ignoring provided secret key. Cannot set custom secret key for authenticated user")

        log.info(msg)
        return self.conn.post(url, json_payload=payload)

    def delete(self, user_id=None, namespace=None, secret_key=None):
        """
        Deletes all secret keys for the specific user. If the user is not provided,
        it will delete the secret keys for the authenticated user. If the user is
        provided and belongs namespace, the namespace must be supplied.

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param user_id: Valid user identifier to get delete the keys from (optional)
        :param namespace: Namespace for the user if the user belongs to a namespace (optional)
        :param secret_key: The secret key to deleted (optional)
        """

        if user_id:
            url = 'object/user-secret-keys/{}/deactivate'.format(user_id)
            msg = "Deleting secret for user '{}'".format(user_id)
        else:
            url = 'object/secret-keys/deactivate'
            msg = "Deleting secret for the authenticated user"

        payload = {}
        if secret_key:
            payload['secret_key'] = secret_key
        if namespace:
            payload['namespace'] = namespace

        log.info(msg)
        return self.conn.post(url, json_payload=payload)



import time

from ecsclient import schemas
from ecsclient.common.exceptions import ECSClientException
from tests import functional


class TestNamespace(functional.BaseTestCase):
    def __init__(self, *args, **kwargs):
        super(TestNamespace, self).__init__(*args, **kwargs)
        self.namespace_1 = "functional-tests-namespace-%s" % int(time.time())
        self.namespace_2 = self.namespace_1 + '_second'
        self.namespace_3 = self.namespace_1 + '_third'

    def setUp(self):
        super(TestNamespace, self).setUp()
        self.client.namespace.create(self.namespace_1)
        self.client.namespace.create(self.namespace_3)

    def tearDown(self):
        super(TestNamespace, self).tearDown()
        for namespace in [self.namespace_1,
                          self.namespace_2,
                          self.namespace_3]:
            try:
                self.client.namespace.delete(namespace)
            except ECSClientException:
                pass

    def test_namespaces_list(self):
        response = self.client.namespace.list()
        self.assertValidSchema(response, schemas.NAMESPACES)

    def test_namespaces_get_one(self):
        response = self.client.namespace.get(self.namespace_1)
        self.assertValidSchema(response, schemas.NAMESPACE)

    def test_namespaces_update(self):
        # Get the namespace and verify the value of one of its attributes
        response = self.client.namespace.get(self.namespace_1)
        self.assertEqual(response['is_stale_allowed'], False)

        # Update the attribute
        self.client.namespace.update(self.namespace_1, is_stale_allowed=True)

        # Get it again and verify that the value was correctly updated
        response = self.client.namespace.get(self.namespace_1)
        self.assertEqual(response['is_stale_allowed'], True)

    def test_namespaces_create(self):
        response = self.client.namespace.create(self.namespace_2)
        self.assertValidSchema(response, schemas.NAMESPACE)
        self.assertEqual(response['name'], self.namespace_2)

    def test_namespaces_delete(self):
        self.client.namespace.delete(self.namespace_3)
        f = self.client.namespace.get
        self.assertRaises(ECSClientException, f, self.namespace_3)



import logging

log = logging.getLogger(__name__)


class Cas(object):
    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_cas_secret(self, uid, namespace=None):
        """
        Gets CAS secret for the specified user and, optionally, a namespace.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
          "user_cas_secret": { "cas_secret": "secret" }
        }

        :param uid: Valid user identifier to get the key from
        :param namespace: Namespace for which to get CAS secret
        """
        log.info("Fetching CAS secret for user ID '{uid}' (namespace '{namespace}')").format(
            uid=uid,
            namespace=namespace
        )
        if namespace:
            url = 'object/user-cas/secret/{namespace}/{uid}'.format(
                uid=uid,
                namespace=namespace
            )
        else:
            url = 'object/user-cas/secret/{uid}'.format(uid=uid)
        return self.conn.get(url)

    def set_cas_secret(self, uid, namespace, secret):
        """
        Creates or updates CAS secret for a specified user.

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param uid: Valid user identifier to update a secret for
        :param namespace: Namespace identifier to associate with the CAS secret
        :param secret: Secret for the user
        """
        payload = {
          "user_cas_secret_param": {
            "namespace": namespace,
            "secret": secret
          }
        }

        log.info("Updating CAS secret for user ID '{}'".format(uid))
        return self.conn.post('object/user-cas/secret/{}'.format(uid), json_payload=payload)

    def delete_cas_secret(self, uid, namespace, secret):
        """
        Deletes CAS secret for a specified user identifier.

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param uid: Valid user identifier to update a secret for
        :param namespace: Namespace identifier to associate with the CAS secret
        :param secret: CAS Secret for the user
        """
        payload = {
          "namespace": namespace,
          "secret": secret
        }

        log.info("Deleting CAS secret for user ID '{}'".format(uid))
        return self.conn.post('object/user-cas/secret/{}/deactivate'.format(uid), json_payload=payload)

    def generate_pea(self, uid, namespace):
        """
        Generates Pool Entry Authorization (PEA) file for specified user

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
          "pea": {
            "-version": "1.0.0",
            "defaultkey": {
              "-name": "wuser1@sanity.local",
              "credential": {
                "-id": "csp1.secret",
                "-enc": "base64",
                "#text": "c2VjcmV0"
              }
            },
            "key": {
              "-type": "cluster",
              "-id": "19999d80-37b2-3111-9c5d-7c053bc73f1a",
              "-name": "wuser1@sanity.local",
              "credential": {
                "-id": "csp1.secret",
                "-enc": "base64",
                "#text": "c2VjcmV0"
              }
            }
          }
        }

        :param uid: Valid user identifier to create PEA file
        :param namespace: Namespace id with CAS cluster
        """
        log.info("Generating Pool Entry Authorization (PEA) file for user ID '{uid}' (namespace '{namespace}')").format(
            uid=uid,
            namespace=namespace
        )
        return self.conn.get('object/user-cas/secret/{}/{}/pea'.format(uid, namespace))

    def get_default_bucket(self, uid, namespace=None):
        """
        Gets default bucket for the specified user identifier and, optionally, the namespace.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            "name": "standalone-bucket"
        }

        :param uid: Valid user identifier from which to get bucket
        :param namespace: Namespace from which to get bucket
        """
        log.info("Getting default bucket for user ID '{uid}' (namespace '{namespace}')").format(
            uid=uid,
            namespace=namespace
        )
        if namespace:
            url = '/object/user-cas/bucket/{namespace}/{uid}'.format(
                uid=uid,
                namespace=namespace
            )
        else:
            url = '/object/user-cas/bucket/{uid}'.format(uid=uid)
        return self.conn.get(url)

    def set_default_bucket(self, uid, namespace, bucket_name):
        """
        Updates default bucket the specified namespace and user identifier

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
          "user_cas_bucket": { "name": "standalone-bucket" }
        }

        :param uid: Valid user identifier to update default bucket
        :param namespace: Namespace required to update default bucket
        :param bucket_name: Name of the default bucket to be set
        """
        log.info("Setting default bucket to '{bucket_name}' for user ID '{uid}' (namespace '{namespace}')").format(
            uid=uid,
            namespace=namespace,
            bucket_name=bucket_name
        )
        payload = {
          "name": bucket_name
        }
        return self.conn.post('object/user-cas/bucket/{namespace}/{uid}'.format(
            uid=uid,
            namespace=namespace),
            json_payload=payload)

    def get_registered_applications(self, namespace):
        """
        Gets the CAS registered applications for a specified namespace.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:
        {}

        :param namespace: Namespace required to get CAS registered applications
        """
        # TODO: Add example JSON response

        log.info("Getting registered CAS applications on namespace '{namespace}'").format(
            namespace=namespace
        )
        return self.conn.get('object/user-cas/applications/{namespace}'.format(namespace=namespace))

    def get_user_metadata(self, uid, namespace):
        """
        Gets the CAS user metadata for the specified namespace and user identifier.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:
        {
            "metadata": {},
            "user_name": "testlogin"
        }

        :param uid: User identifier for which to get metadata
        :param namespace: Namespace required to get metadata
        """

        log.info("Getting CAS user metadata for user ID '{uid}' (namespace '{namespace}')").format(
            uid=uid,
            namespace=namespace
        )
        return self.conn.get('object/user-cas/metadata/{namespace}/{uid}'.format(
            uid=uid,
            namespace=namespace))



import logging

log = logging.getLogger(__name__)


class Capacity(object):

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_cluster_capacity(self, storage_pool_id=None):
        """
        Gets the capacity of the cluster. The details includes the provisioned
        capacity in GB and available capacity in GB.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'totalFree_gb': 1272085,
            u'totalProvisioned_gb': 2578400
        }

        :param storage_pool_id: Storage pool identifier for which to retrieve
        capacity (optional)
        """
        if storage_pool_id:
            log.info("Getting capacity of storage pool '{0}'".format(storage_pool_id))
            return self.conn.get(url='object/capacity/{0}'.format(storage_pool_id))
        else:
            log.info("Getting capacity of storage cluster")
            return self.conn.get(url='object/capacity')



from ecsclient import schemas
from tests import functional


class TestCapacity(functional.BaseTestCase):

    def test_get_cluster_capacity(self):
        response = self.client.capacity.get_cluster_capacity()
        self.assertValidSchema(response, schemas.CAPACITY)

    def test_get_cluster_capacity_storage_pool(self):
        sps = self.client.storage_pool.list()
        response = self.client.capacity.get_cluster_capacity(storage_pool_id=sps['varray'][0]['id'])
        self.assertValidSchema(response, schemas.CAPACITY)



# Standard lib imports
import logging

# Third party imports
# None

# Project level imports
# None


log = logging.getLogger(__name__)


class ConfigurationProperties(object):

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_properties(self, category='ALL'):
        """
        Gets the configuration properties for the specified category. If the
        category is not provided, then properties of all categories will
        be retrieved

        :param category:  The category for which to retrieve configuration
        properties. If this is not provided it defaults to "ALL"

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'allProperties': {
                u'user_scope': u'GLOBAL'
            },
            u'properties': {
                u'user_scope': u'GLOBAL'
            },
            u'empty': False
        }

        """
        log.info("Getting configurations in category '{0}'".format(category))
        return self.conn.get(
            'config/object/properties?category={0}'.format(category))

    def get_properties_metadata(self):
        """
        Gets the meta data for each of the configuration properties.
        Metadata includes the possible values for the property, a description
        of the property, whether a reboot is required when it is changed, etc.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'metadata': {
                u'user_scope': {
                    u'reconfigRequired': False,
                    u'description': u'Declaresthescopeforusernameuniqueness',
                    u'userMutable': False,
                    u'minLen': 0,
                    u'userConfigurable': False,
                    u'defaultValue': None,
                    u'controlNodeOnly': False,
                    u'value': u'GLOBAL',
                    u'allowedValues': [
                        u'NAMESPACE',
                        u'GLOBAL'
                    ],
                    u'maxLen': 65534,
                    u'defaultValueMetaData': u'GLOBAL',
                    u'hidden': False,
                    u'label': u'Userscope',
                    u'type': u'string',
                    u'rebootRequired': False,
                    u'advanced': False
                }
            }
        }

        """
        log.info("Getting metadata properties")
        return self.conn.get('config/object/properties/metadata')

    def set_properties(self):
        """
        Sets the configuration properties for the system.

        Properties are:
        user-scope: defines whether user accounts should be treated as GLOBAL
        or NAMESPACE.

        In GLOBAL scope, users are global and are can be shared across
        namespaces. In this case, the default namespace associated with a user
        determines the namespace for object operations and there is no need to
        supply a namespace for an operation.

        If the user scope is NAMESPACE, a user is associated with a namespace,
        so there might be more than user with the same name, each associated
        with a different namespace. In NAMESPACE mode a namespace must be
        provided for every operation. Must be set before the first user is
        created.

        The default setting is GLOBAL.
        """
        log.error("Configuration properties modification not supported")
        raise NotImplementedError



import logging

log = logging.getLogger(__name__)


class Alerts(object):

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_alerts(self, namespace=None, start_time=None, end_time=None, marker=None, limit=None,
                   severity=None, type=None, acknowledged=None):
        """
        Gets the list of alerts with optional filters

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        {
            ...
        }

        :param namespace: Namespace for which alerts should be listed
        :param start_time: Start time for listing alerts
        :param end_time: End time for listing alerts
        :param marker: Reference to last alert returned
        :param limit: Number of alerts requested in current fetch
        :param severity: Severity of alerts to be listed
        :param type: Type of alerts to be listed
        :param acknowledged: Boolean to filter by acknowledgement
        """

        filters = {}
        if namespace:
            filters['namespace'] = namespace
        if start_time:
            filters['start_time'] = start_time
        if end_time:
            filters['end_time'] = end_time
        if marker:
            filters['marker'] = marker
        if limit:
            filters['limit'] = limit
        if severity:
            filters['severity'] = severity
        if type:
            filters['type'] = type
        if acknowledged:
            filters['acknowledged'] = acknowledged

        log.info("Getting alerts with filters: %s", (filters,))
        return self.conn.get(url='vdc/alerts', params=filters)



from ecsclient import schemas
from tests import functional


class TestAlerts(functional.BaseTestCase):
    def test_alerts_no_params(self):
        response = self.client.alerts.get_alerts()
        # TODO: complete the schema with alert element validation
        self.assertValidSchema(response, schemas.ALERTS)

    def test_alerts_with_params(self):
        self.skipTest("Need to test it with params")



# coding=utf-8
import logging

log = logging.getLogger(__name__)


class PasswordGroup(object):
    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get(self, user_id, namespace=None):
        """
        Gets all Swift user groups for a specified user identifier.
        If namespace is provided then returns only groups for the specified namespace.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            "groups_list": [
                "admin"
            ]
        }

        :param user_id: Username for which group names should be returned.
        :param namespace: Namespace limiting returned groups (optional).
        """
        msg = "Getting Swift groups for user '{}'".format(user_id)
        url = 'object/user-password/{}'.format(user_id)

        if namespace:
            url += '/{}'.format(namespace)
            msg += " in namespace '{}'".format(namespace)

        log.info(msg)
        return self.conn.get(url=url)

    def create(self, user_id, password, groups_list, namespace=None):
        """
        Creates password and group for a specific user.

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param user_id: Valid user identifier to create a key for. If not provided, the authenticated
        user will be used instead.
        :param namespace: Namespace for the user if the user belongs to a namespace.
        :param password: Swift password associated with this user.
        :param groups_list: List of Swift groups with which to associate this user. If user is a member
        of the "admin" group, user will be able to perform all container operations.
        If a member of any other group, authorization will depend on the access that is set on the container.
        """

        url = 'object/user-password/{}'.format(user_id)
        msg = "Creating Swift password for user '{}' in namespace '{}'".format(user_id, namespace)

        payload = {'namespace': namespace,
                   'password': password,
                   'groups_list': groups_list}

        log.info(msg)
        return self.conn.put(url, json_payload=payload)

    def update(self, user_id, password, groups_list, namespace=None):
        """
        Updates password and group information for a specific user identifier.

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param user_id: Valid user identifier to create a key for. If not provided,
        the authenticated user will be used instead.
        :param namespace: Namespace for the user if the user belongs to a namespace.
        :param password: Swift password associated with this user.
        :param groups_list: List of Swift groups with which to associate this user. If user is a member
        of the "admin" group, user will be able to perform all container operations.
        If a member of any other group, authorization will depend on the access that is set on the container.
        """

        url = 'object/user-password/{}'.format(user_id)
        msg = "Updating Swift password for user '{}' in namespace '{}'".format(user_id, namespace)

        payload = {'namespace': namespace,
                   'password': password,
                   'groups_list': groups_list}

        log.info(msg)
        return self.conn.post(url, json_payload=payload)

    def delete(self, user_id, namespace=None):
        """
        Deletes password group for a specified user.

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 204 No Response

        :param user_id: Valid user identifier to get delete the keys from.
        :param namespace: Namespace for the user if the user belongs to a namespace.
        """

        url = 'object/user-password/{}/deactivate'.format(user_id)
        msg = "Deleting Swift password for user '{}' in namespace '{}'".format(user_id, namespace)

        payload = {'namespace': namespace}

        log.info(msg)
        return self.conn.post(url, json_payload=payload)



import time

from ecsclient import schemas
from ecsclient.common.exceptions import ECSClientException
from tests import functional


class TestReplicationGroup(functional.BaseTestCase):

    def setUp(self):
        super(TestReplicationGroup, self).setUp()
        # Initialize the resource names
        self.storage_pool_1 = "functional-tests-storagepool-%s" % int(time.time())
        self.replication_group_1 = "functional-tests-replicationgroup-%s" % int(time.time())
        self.replication_group_1_id = "placeholder"
        self.replication_group_2 = self.replication_group_1 + '_second'
        self.replication_group_2_id = "placeholder"

        # Get the local VDC
        r = self.client.vdc.get_local()
        self.vdc_1_id = r['id']

        # Create a Storage Pool
        r = self.client.storage_pool.create(self.storage_pool_1)
        self.storage_pool_1_id = r['id']

        # Create a Replication Group
        zone_mappings = [(
            self.vdc_1_id,
            self.storage_pool_1_id
        )]
        r = self.client.replication_group.create(self.replication_group_1, zone_mappings)
        self.replication_group_1_id = r['id']

    def tearDown(self):
        super(TestReplicationGroup, self).tearDown()

        try:
            self.client.storage_pool.delete(self.storage_pool_1_id)
        except ECSClientException:
            pass

        for replication_group_id in [self.replication_group_1_id,
                                     self.replication_group_2_id]:
            try:
                self.client.replication_group.delete(replication_group_id)
            except ECSClientException:
                pass

    def test_replication_group_list(self):
        response = self.client.replication_group.list()
        self.assertValidSchema(response, schemas.REPLICATION_GROUPS)

    def test_replication_group_get_one(self):
        response = self.client.replication_group.get(self.replication_group_1_id)
        self.assertValidSchema(response, schemas.REPLICATION_GROUP)
        self.assertEqual(response['id'], self.replication_group_1_id)

    def test_replication_group_update(self):
        new_name = self.replication_group_1 + '_updated'
        response = self.client.replication_group.get(self.replication_group_1_id)
        self.assertNotEqual(response['name'], new_name)
        self.assertNotEqual(response['description'], new_name + ' description')
        self.assertTrue(response['isAllowAllNamespaces'])
        self.assertFalse(response['enable_rebalancing'])

        self.client.replication_group.update(self.replication_group_1_id,
                                             name=new_name,
                                             description=new_name + ' description',
                                             allow_all_namespaces=False,
                                             enable_rebalancing=True)

        response = self.client.replication_group.get(self.replication_group_1_id)
        self.assertEqual(response['name'], new_name)
        self.assertEqual(response['description'], new_name + ' description')
        self.assertFalse(response['isAllowAllNamespaces'])
        self.assertTrue(response['enable_rebalancing'])

    def test_replication_group_create(self):
        zone_mappings = [(
            self.vdc_1_id,
            self.storage_pool_1_id
        )]
        response = self.client.replication_group.create(self.replication_group_2,
                                                        zone_mappings,
                                                        description=self.replication_group_2 + " description",
                                                        allow_all_namespaces=False,
                                                        is_full_rep=False,
                                                        enable_rebalancing=True)
        self.assertValidSchema(response, schemas.REPLICATION_GROUP)
        self.assertEqual(response['name'], self.replication_group_2)
        self.assertEqual(response['description'], self.replication_group_2 + " description")
        self.assertFalse(response['isAllowAllNamespaces'])
        self.assertFalse(response['isFullRep'])
        self.assertTrue(response['enable_rebalancing'])
        self.assertEqual(response['varrayMappings'][0]['name'], self.vdc_1_id)
        self.assertEqual(response['varrayMappings'][0]['value'], self.storage_pool_1_id)

    def test_replication_group_delete(self):
        self.client.replication_group.delete(self.replication_group_1_id)
        f = self.client.replication_group.get
        self.assertRaises(ECSClientException, f, self.replication_group_1_id)



import json
import logging
import os

import requests

from ecsclient.authentication import Authentication
from ecsclient.common.exceptions import ECSClientException
from ecsclient.common.token_request import TokenRequest

# Suppress the insecure request warning
# https://urllib3.readthedocs.org/en/
# latest/security.html#insecurerequestwarning
requests.packages.urllib3.disable_warnings()

# Initialize logger
log = logging.getLogger(__name__)
log.addHandler(logging.NullHandler())


class Client(object):

    def __init__(self, username=None, password=None, token=None,
                 ecs_endpoint=None, token_endpoint=None, verify_ssl=False,
                 token_path='/tmp/ecsclient.tkn',
                 request_timeout=15.0, cache_token=True, override_header=None):
        """
        Creates the ECSClient class that the client will directly work with

        :param username:
        The username to fetch a token
        :param password: The password to fetch a token
        :param token: Supply a valid token to use instead of username/password
        :param ecs_endpoint: The URL where ECS is located
        :param token_endpoint: The URL where the ECS login is located
        :param verify_ssl: Verify SSL certificates
        :param token_path: Path to the cached token file
        :param request_timeout: How long to wait for ECS to respond
        :param cache_token: Whether to cache the token, by default this is true
        you should only switch this to false when you want to directly fetch
        a token for a user
        :param override_header: X-EMC-Override header value into API calls
        """
        if not ecs_endpoint:
            raise ECSClientException("Missing 'ecs_endpoint'")

        self.token_endpoint = token_endpoint

        if token_endpoint:
            if not (username and password):
                raise ECSClientException("'token_endpoint' provided but missing ('username','password')")
            self.token_endpoint = self.token_endpoint.rstrip('/')
        else:
            if not (token or os.path.isfile(token_path)):
                raise ECSClientException("'token_endpoint' not provided and missing 'token'|'token_path'")

        self.override_header = override_header
        self.username = username
        self.password = password
        self.token = token
        self.ecs_endpoint = ecs_endpoint.rstrip('/')
        self.verify_ssl = verify_ssl
        self.token_path = token_path
        self.request_timeout = request_timeout
        self.cache_token = cache_token
        self._session = requests.Session()
        self._token_request = TokenRequest(
            username=self.username,
            password=self.password,
            ecs_endpoint=self.ecs_endpoint,
            token_endpoint=self.token_endpoint,
            verify_ssl=self.verify_ssl,
            token_path=self.token_path,
            request_timeout=self.request_timeout,
            cache_token=self.cache_token)

        # Authentication
        self.authentication = Authentication(self)

    def get_token(self):
        """
        Get a token directly back, typically you want to set the cache_token
        param for ecsclient to false for this call.

        :return: A valid token or an ecsclient exception
        """
        return self._token_request.get_new_token()

    def get_current_token(self):
        """
        Get the current token in use. None if the client is logged out or not yet logged in
        """
        return self._token_request.token

    def remove_cached_token(self):
        """
        Remove the cached token file, this is useful if you switch users
        and want to use a different token
        """
        self.token = None
        self._token_request.token = None

        if os.path.isfile(self.token_path):
            log.debug("Removing cached token '{0}'".format(self.token_path))
            os.remove(self.token_path)

    def _fetch_headers(self):
        token = self.token if self.token else self._token_request.get_token()
        headers = {'Accept': 'application/json',
                   'Content-Type': 'application/json',
                   'x-sds-auth-token': token}
        if self.override_header is not None:
            headers['X-EMC-Override'] = self.override_header
        return headers

    def _construct_url(self, path):
        url = '{0}/{1}'.format(self.ecs_endpoint, path)
        log.debug('Constructed URL as: {0}'.format(url))
        return url

    def get(self, url, params=None):
        return self._request(url, params=params)

    def post(self, url, json_payload='{}'):
        return self._request(url, json_payload, http_verb='POST')

    def put(self, url, json_payload='{}'):
        return self._request(url, json_payload, http_verb='PUT')

    def delete(self, url, params=None):
        return self._request(url, params=params, http_verb='DELETE')

    def _request(self, url, json_payload='{}', http_verb='GET', params=None):
        json_payload = json.dumps(json_payload)

        try:
            if http_verb == "PUT":
                req = self._session.put(
                    self._construct_url(url),
                    verify=self.verify_ssl,
                    headers=self._fetch_headers(),
                    timeout=self.request_timeout,
                    data=json_payload)
            elif http_verb == 'POST':
                req = self._session.post(
                    self._construct_url(url),
                    verify=self.verify_ssl,
                    headers=self._fetch_headers(),
                    timeout=self.request_timeout,
                    data=json_payload)
            elif http_verb == 'DELETE':
                # Need to follow up - if 'accept' is in the headers
                # delete calls are not working because ECS 2.0 is returning
                # XML even if JSON is specified
                headers = self._fetch_headers()
                del headers['Accept']

                req = self._session.delete(
                    self._construct_url(url),
                    verify=self.verify_ssl,
                    headers=headers,
                    timeout=self.request_timeout,
                    params=params)
            else:  # Default to GET
                req = self._session.get(
                    self._construct_url(url),
                    verify=self.verify_ssl,
                    headers=self._fetch_headers(),
                    timeout=self.request_timeout,
                    params=params)

            # Because some delete actions in the API return HTTP/1.1 204 No Content
            if not (200 <= req.status_code < 300):
                log.error("Status code NOT OK")
                raise ECSClientException.from_response(req)
            try:
                return req.json()
            except ValueError:
                return req.text

        except requests.ConnectionError as conn_err:
            msg = 'Connection error: {0}'.format(conn_err.args)
            log.error(msg)
            raise ECSClientException(message=msg)
        except requests.HTTPError as http_err:
            msg = 'HTTP error: {0}'.format(http_err.args)
            log.error(msg)
            raise ECSClientException(message=msg)
        except requests.RequestException as req_err:
            msg = 'Request error: {0}'.format(req_err.args)
            log.error(msg)
            raise ECSClientException(message=msg)



from ecsclient import schemas
from tests import functional
from tests.functional import helper


class TestVdcKeystore(functional.BaseTestCase):

    def test_vdc_keystore_get(self):
        response = self.client.vdc_keystore.get()
        self.assertValidSchema(response, schemas.VDC_KEYSTORE)

    def test_vdc_keystore_set(self):
        private_key = helper.get_sample_private_key()
        certificate = helper.get_sample_certificate()
        response = self.client.vdc_keystore.set(private_key, certificate)
        self.assertValidSchema(response, schemas.VDC_KEYSTORE)
        # Can't validate the certificate and private key at this point
        # because it does not take effect until the ECS is manually restarted
        # self.assertSameCertificate(certificate, response['chain'])



# Standard lib imports
import logging

# Third party imports
# None

# Project level imports
# None


log = logging.getLogger(__name__)


class BaseUrl(object):

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_all_configured_base_urls(self):
        """
        Lists all configured Base URLs.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'base_url': [
                {
                    u'link': {
                        u'href': u'/object/baseurl/urn: ObjectBaseUrl:
                                    6c74e6fb-a2a1-4386-bc25-b4399a6e74ce',
                        u'rel': u'self'
                    },
                    u'name': u'US1',
                    u'id': u'urn: ObjectBaseUrl:
                                6c74e6fb-a2a1-4386-bc25-b4399a6e74ce'
                },
                {
                    u'link': {
                        u'href': u'/object/baseurl/urn: ObjectBaseUrl:
                                    70f63a6f-25be-432c-875f-61c4a3953c42',
                        u'rel': u'self'
                    },
                    u'name': u'DefaultBaseUrl',
                    u'id': u'urn: ObjectBaseUrl:
                                70f63a6f-25be-432c-875f-61c4a3953c42'
                },
                {
                    u'link': {
                        u'href': u'/object/baseurl/urn: ObjectBaseUrl:
                                    72863b71-27a8-4917-8df0-cc84d3bdfe98',
                        u'rel': u'self'
                    },
                    u'name': u'US2',
                    u'id': u'urn: ObjectBaseUrl:
                                72863b71-27a8-4917-8df0-cc84d3bdfe98'
                }
            ]
        }
        """
        log.info("Getting all Base URLs")
        return self.conn.get(url='object/baseurl')

    def get_base_url(self, base_url_id):
        """
        Gets details for the specified Base URL.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'remote': None,
            u'name': u'US1',
            u'tags': [

            ],
            u'global': None,
            u'baseUrlNamespaceInHostFlag': False,
            u'baseurl': u'os-us1.rraas-ops.com',
            u'vdc': None,
            u'link': {
                u'href': u'/object/baseurl/urn: ObjectBaseUrl:
                            6c74e6fb-a2a1-4386-bc25-b4399a6e74ce',
                u'rel': u'self'
            },
            u'id': u'urn: ObjectBaseUrl: 6c74e6fb-a2a1-4386-bc25-b4399a6e74ce'
        }

        :param base_url_id: Base URL identifier for the Base URL that needs to
        be retrieved
        """
        log.info("Getting Base URL '{0}'".format(base_url_id))
        return self.conn.get(url='object/baseurl/{0}'.format(base_url_id))

    def create_base_url(self, name, base_url, is_namespace_in_host=True):
        """
        Creates a Base URL with the given details.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        {
            u'remote': None,
            u'name': u'TestBaseURL',
            u'tags': [

            ],
            u'global': None,
            u'baseUrlNamespaceInHostFlag': False,
            u'baseurl': u'test.com',
            u'vdc': None,
            u'link': {
                u'href': u'/object/baseurl/urn: ObjectBaseUrl:
                            19c391eb-37f4-4c65-a7a9-474668f71607',
                u'rel': u'self'
            },
            u'id': u'urn: ObjectBaseUrl: 19c391eb-37f4-4c65-a7a9-474668f71607'
        }

        :param name: Name for this Base-URL
        :param base_url: Base URL to be used
        :param is_namespace_in_host: Set true if namespace is in host, false
        otherwise
        """
        payload = {
            "name": name,
            "base_url": base_url,
            "is_namespace_in_host": is_namespace_in_host
        }
        log.info("Creating Base URL '{0}': {1}".format(name, payload))
        return self.conn.post(url='object/baseurl', json_payload=payload)

    def delete_base_url(self, base_url_id):
        """
        Updates the owner for the specified bucket.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param base_url_id: Base URL identifier that needs to be deleted
        """
        log.info("Deleting Base URL '{0}'".format(base_url_id))

        return self.conn.post(
            url='object/baseurl/{0}/deactivate'.format(base_url_id))

    def modify_base_url(self, base_url_id, name, base_url,
                        is_namespace_in_host=True):
        """
        Updates the Base URL for the specified Base URL identifier.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param base_url_id: Base URL identifier that needs to be updated
        :param name: Name for this Base-URL
        :param base_url: Base URL to be used
        :param is_namespace_in_host: Set true if namespace is in host, false
        otherwise
        """
        payload = {
            "name": name,
            "base_url": base_url,
            "is_namespace_in_host": is_namespace_in_host
        }

        log.info("Updating Base URL '{0}': {1}".format(base_url_id, payload))

        return self.conn.put(
            url='object/baseurl/{0}'.format(base_url_id), json_payload=payload)



from ecsclient import schemas
from tests import functional
from tests.functional import helper


class TestCertificate(functional.BaseTestCase):
    def test_get_certificate(self):
        response = self.client.certificate.get_certificate_chain()
        self.assertValidSchema(response, schemas.CERTIFICATE)

    def test_set_certificate(self):
        # First, set a self-signed certificate
        ip_addresses = ['10.0.0.1']
        response = self.client.certificate.set_certificate_chain(
            selfsigned=True,
            ip_addresses=ip_addresses)
        self.assertValidSchema(response, schemas.CERTIFICATE)

        # Then, provide a private key and a certificate
        private_key = helper.get_sample_private_key()
        certificate = helper.get_sample_certificate()
        response = self.client.certificate.set_certificate_chain(
            private_key=private_key,
            certificate_chain=certificate)
        self.assertValidSchema(response, schemas.CERTIFICATE)
        self.assertSameCertificate(certificate, response['chain'])



import unittest

import mock

from ecsclient import baseclient
from ecsclient import v2, v3
from ecsclient.client import Client
from ecsclient.common.exceptions import ECSClientException


class TestEcsClient(unittest.TestCase):
    def test_verify_attributes(self):
        c = baseclient.Client(username='someone',
                              password='password',
                              ecs_endpoint='http://127.0.0.1:4443',
                              token_endpoint='http://127.0.0.1:4443/login')
        attributes = ['token_endpoint',
                      'username',
                      'password',
                      'token',
                      'ecs_endpoint',
                      'verify_ssl',
                      'token_path',
                      'request_timeout',
                      'cache_token',
                      '_session',
                      '_token_request',
                      'authentication']
        for attr in attributes:
            self.assertTrue(hasattr(c, attr))

    def test_client_without_version(self):
        with self.assertRaises(RuntimeError) as error:
            Client(username='user',
                   password='password',
                   ecs_endpoint='https://192.168.1.10',
                   token_endpoint='https://192.168.10/login')
        exception = error.exception
        self.assertIn('Please provide the API version', str(exception))

    def test_client_unsupported_version(self):
        with self.assertRaises(RuntimeError) as error:
            Client(version='10',
                   username='user',
                   password='password',
                   ecs_endpoint='https://192.168.1.10',
                   token_endpoint='https://192.168.10/login')
        exception = error.exception
        self.assertEqual("No client available for version '10'", str(exception))

    def test_client_without_ecs_endpoint(self):
        with self.assertRaises(ECSClientException) as error:
            Client(version='3',
                   username='user',
                   password='password',
                   token_endpoint='https://192.168.10/login')
        exception = error.exception.message
        self.assertEqual("Missing 'ecs_endpoint'", str(exception))

    @mock.patch('ecsclient.baseclient.os.path.isfile')
    def test_client_without_token_endpoint(self, mock_isfile):
        mock_isfile.return_value = False
        with self.assertRaises(ECSClientException) as error:
            Client(version='3',
                   username='user',
                   password='password',
                   ecs_endpoint='https://192.168.1.10')
        exception = error.exception.message
        mock_isfile.assert_called_with('/tmp/ecsclient.tkn')
        self.assertEqual("'token_endpoint' not provided and missing 'token'|'token_path'", str(exception))

    def test_client_without_credentials(self):
        with self.assertRaises(ECSClientException) as error:
            Client(version='3',
                   ecs_endpoint='https://192.168.1.10',
                   token_endpoint='https://192.168.10/login')
        exception = error.exception.message
        self.assertEqual("'token_endpoint' provided but missing ('username','password')", str(exception))

    def test_client_v3_class(self):
        c = Client(version='3',
                   username='user',
                   password='password',
                   ecs_endpoint='https://192.168.1.10',
                   token_endpoint='https://192.168.10/login')
        self.assertIsInstance(c, v3.client.Client, 'Instance is not a v3 client class')

    def test_client_v2_class(self):
        c = Client(version='2',
                   username='user',
                   password='password',
                   ecs_endpoint='https://192.168.1.10',
                   token_endpoint='https://192.168.10/login')
        self.assertIsInstance(c, v2.client.Client, 'Instance is not a v2 client class')

    def test_client_init_with_credentials(self):
        c = Client(version='3',
                   username='user',
                   password='password',
                   token_endpoint='https://192.168.10/login',
                   ecs_endpoint='https://192.168.1.10')
        self.assertTrue(hasattr(c, 'username'))
        self.assertTrue(hasattr(c, 'password'))
        self.assertTrue(hasattr(c, 'token_endpoint'))

    def test_client_init_with_token(self):
        c = Client(version='3',
                   token='1234567890',
                   ecs_endpoint='https://192.168.1.10')
        self.assertTrue(hasattr(c, 'token'))

    @mock.patch('ecsclient.baseclient.os.path.isfile')
    def test_client_init_with_token_path(self, mock_isfile):
        mock_isfile.return_value = True
        c = Client(version='3',
                   token_path='/tmp/mytoken.tkn',
                   ecs_endpoint='https://192.168.1.10')
        self.assertTrue(hasattr(c, 'token_path'))
        mock_isfile.assert_called_with('/tmp/mytoken.tkn')



import json

import testtools
from requests_mock.contrib import fixture
from six import string_types

from ecsclient.client import Client


class TestHttp(testtools.TestCase):

    TEST_URL = 'http://127.0.0.1:4443/hi'

    def __init__(self, *args, **kwargs):
        super(TestHttp, self).__init__(*args, **kwargs)
        self.client = Client('3',
                             username='user',
                             password='password',
                             token='token',
                             ecs_endpoint='http://127.0.0.1:4443',
                             token_endpoint='http://127.0.0.1:4443/login')

    def setUp(self):
        super(TestHttp, self).setUp()
        self.requests_mock = self.useFixture(fixture.Fixture())

    def test_get_request_plaintext_resp(self):
        self.requests_mock.register_uri('GET', self.TEST_URL, text='resp')

        body = self.client.get('hi', params={'key1': 'value1'})

        self.assertIsInstance(body, string_types)
        self.assertEqual('resp', body)
        self.assertEqual(self.requests_mock.last_request.method, 'GET')
        self.assertEqual(self.requests_mock.last_request.url, self.TEST_URL + '?key1=value1')
        self.assertEqual(self.requests_mock.last_request.headers['x-sds-auth-token'], 'token')

    def test_get_request_json_resp(self):
        self.requests_mock.register_uri('GET', self.TEST_URL, text='{"key2": "value2"}')

        body = self.client.get('hi', params={'key1': 'value1'})

        self.assertIsInstance(body, dict)
        self.assertEqual(body['key2'], "value2")
        self.assertEqual(self.requests_mock.last_request.method, 'GET')
        self.assertEqual(self.requests_mock.last_request.url, self.TEST_URL + '?key1=value1')
        self.assertEqual(self.requests_mock.last_request.headers['x-sds-auth-token'], 'token')

    def test_post_request(self):
        self.requests_mock.register_uri('POST', self.TEST_URL, text=None)

        payload = {'key1': 'value1', 'key2': 'value2'}
        self.client.post('hi', json_payload=payload)

        self.assertEqual(self.requests_mock.last_request.method, 'POST')
        self.assertEqual(json.loads(self.requests_mock.last_request.text), payload)
        self.assertEqual(self.requests_mock.last_request.url, self.TEST_URL)
        self.assertEqual(self.requests_mock.last_request.headers['x-sds-auth-token'], 'token')

    def test_put_request(self):
        self.requests_mock.register_uri('PUT', self.TEST_URL, text=None)

        payload = {'key1': 'value1', 'key2': 'value2'}
        self.client.put('hi', json_payload=payload)

        self.assertEqual(self.requests_mock.last_request.method, 'PUT')
        self.assertEqual(json.loads(self.requests_mock.last_request.text), payload)
        self.assertEqual(self.requests_mock.last_request.url, self.TEST_URL)
        self.assertEqual(self.requests_mock.last_request.headers['x-sds-auth-token'], 'token')

    def test_delete_request(self):
        self.requests_mock.register_uri('DELETE', self.TEST_URL, text=None)

        self.client.delete('hi')

        self.assertEqual(self.requests_mock.last_request.method, 'DELETE')
        self.assertEqual(self.requests_mock.last_request.url, self.TEST_URL)
        self.assertEqual(self.requests_mock.last_request.headers['x-sds-auth-token'], 'token')


# Standard lib imports
import logging

# Third party imports
# None

# Project level imports
# None


log = logging.getLogger(__name__)


class AuthenticationProvider(object):

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_authentication_providers(self):
        """
        Lists the configured authentication providers.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'authnprovider': [
                {
                    u'id': u'urn: AuthProvider:
                            fe4843f3-71eb-4a86-b1bd-806f4275998b',
                    u'name': u'internal'
                }
            ]
        }
        """
        log.info('Getting all authentication providers')
        return self.conn.get(url='vdc/admin/authnproviders')

    def get_authentication_provider(self, auth_provider_id):
        """
        Lists the configured authentication providers.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'search_scope': u'ONELEVEL',
            u'group_whitelist_values': [

            ],
            u'remote': None,
            u'name': u'internal',
            u'tags': [

            ],
            u'manager_dn': u'uid=admin,
            cn=users,
            cn=accounts,
            dc=osaas-lab,
            dc=rcsops,
            dc=com',
            u'max_page_size': 0,
            u'search_base': u'cn=users,
            cn=accounts,
            dc=osaas-lab,
            dc=rcsops,
            dc=com',
            u'global': None,
            u'group_attribute': u'CN',
            u'server_urls': [
                u'ldaps: //192.29.62.18: 636/',
                u'ldaps: //192.29.62.83: 636/',
                u'ldaps: //192.29.62.84: 636/',
                u'ldaps: //192.29.62.19: 636/'
            ],
            u'vdc': None,
            u'disable': False,
            u'mode': u'ldap',
            u'domains': [
                u'tenantadmins',
                u'internal'
            ],
            u'search_filter': u'uid=%U',
            u'id': u'urn: AuthProvider: 317843ad-71eb-4a86-b1bd-806f4275008a',
            u'description': u'internal'
        }

        param: auth_provider_id: Authentication provider identifier URN
        """
        log.info("Getting auth provider '{0}'".format(auth_provider_id))

        return self.conn.get(
            url='vdc/admin/authnproviders/{0}'.format(auth_provider_id))



ERROR_CODES = {
    "127": "Exceeding assignment limit",
    "1002": "URI parameter is invalid",
    "1003": "Unsupported or missing media type",
    "1004": "Request parameter cannot be found",
    "1005": "Required parameter is missing or empty",
    "1006": "Request parameter is inactive and marked for deletion",
    "1007": "Method not supported",
    "1008": "Parameter was provided but invalid",
    "1009": "Bad headers",
    "1010": "Resource marked For deletion",
    "1011": "Unable to use provided VPool",
    "1012": "Unable to make requested change",
    "1013": "Bad request body",
    "1014": "Parameter value not within range",
    "1015": "Resource already exists",
    "1016": "Invalid zone specified",
    "1017": "Unable to register system",
    "1018": "Not registered",
    "1019": "Resource not found",
    "1020": "Resource is being referenced",
    "1021": "Unable to find a suitable placement to handle the request",
    "1022": "Already registered",
    "1023": "Version is not supported",
    "1025": "Unable to delete resource",
    "1026": "Insufficient permissions for user",
    "1028": "Unsupported role assignment",
    "1029": "Time bucket parameter invalid",
    "1030": "Virtual pool change would be disruptive",
    "1031": "Exceeding limit",
    "1032": "An error occurred during quota validation for provisioning",
    "1033": "Unknown RecoverPoint configuration specified",
    "1034": "An error occurred while finding a suitable placement to handle the request",
    "1035": "Attachment(s) size is more than maximum allowed",
    "1036": "Invalid volume type",
    "1037": "Invalid protection virtual pool configuration.",
    "1038": "Invalid file share name specified",
    "1039": "Invalid Network Configuration",
    "1040": "Invalid virtual pool",
    "1041": "There is no download in progress",
    "1042": "Operation not supported on ingested volumes",
    # "1042": "Invalid maximum continuous copies",
    "1043": "The high availability for the continuous copies virtual pool is invalid",
    "1044": "Invalid continuous copies vpool",
    "1045": "Invalid action",
    "1046": "Virtual pool in use as continuous copies virtual pool",
    "1047": "API not been initialized",
    "1048": "Unable to find mirror virtual pool",
    "1049": "Parameter was not provided",
    "2000": "Unable to find entity in request URL",
    "2001": "URI parameter is inactive and marked for deletion",
    "3000": "This operation is forbidden for this resource using the specified credentials",
    "3001": "This operation is forbidden due to license check failure.",
    "4000": "Invalid credentials or authentication token provided to access to this resource",
    "5001": "An error occurred in the controller during a file operation",
    "5002": "An error occurred in the controller during a file storage device connection operation",
    "6000": "Unable to connect to the service. The service is unavailable, try again later",
    "6001": "Download initializing",
    "7000": "An error occurred in the API Service",
    "7001": "An error occurred when getting the Jaxb context",
    "7002": "An error occurred in the Audit Log Service",
    "7003": "An error occurred in the Metering service",
    "7004": "Event retrieval error",
    "7005": "An occurred during the Ingestion Request",
    "7006": "An error occurred during deletion of the RP volume",
    "7007": "An error occurred during RP volume creation",
    "7008": "Error occurred during downloading image",
    "8000": "An error occurred in the database",
    "8001": "Unable to find the object with the given id",
    "8003": "Failed to write to database",
    "8004": "Failed to read from database",
    "8005": "Failed to delete from database",
    "8006": "Failed to query from database",
    "8007": "Invalid annotation",
    "8500": "Unable to connect to the database service",
    "8501": "Dummy failure for testing",
    "9000": "Unable to queue the job",
    "9001": "An error occurred in the coordinator",
    "9002": "Error occurred while decoding from coordinator",
    "9003": "Invalid repository information",
    "9004": "Not connectable endpoint",
    "9500": "Unable to queue the job",
    "9501": "The coordinator was unable to locate the service",
    "10000": "A security error has occurred.",
    "10001": "An error occurred while encoding/decoding of tokens",
    "10004": "Required parameter is missing or empty",
    "10005": "An error occurred while verifying the service signature",
    "10500": "A service which is required to complete the security request is unavailable",
    "10501": "ViPR keystore operation is unavailable",
    "11000": "Unable to schedule job",
    "11001": "Unable to locate device controller",
    "11002": "Unable to scan job",
    "11003": "Unable to monitor job",
    "12002": "An error occurred while metering storage devices",
    "12003": "An error occurred while monitoring storage devices",
    "12005": "URI parameter is invalid",
    "12006": "Controller Entity is marked for deletion",
    "12007": "An error occurred when updating the EndPoints",
    "12008": "Attempt to use an unknown transport zone",
    "12009": "Unable to find the controller entity",
    "12011": "An error occurred in getting the Block Object's Native ID",
    "14000": "An error occurred in a workflow step",
    "14002": "The workflow step has been cancelled",
    "15000": "Unable to dispatch to a controller",
    "30000": "Cluster state is not stable",
    "30001": "Error occurred while releasing lock",
    "30002": "Object is null or empty",
    "30003": "Error occurred while writing",
    "30004": "Error occurred while reading",
    "30005": "Error occurred while creating object",
    "30006": "Error occurred while retrieving object",
    "30007": "Error occurred while setting object",
    "30008": "Error occurred while updating object",
    "30009": "Self test wait to complete",
    "30010": "Self test error occurred",
    "30011": "Error occurred while executing upload install",
    "30012": "Error occurred while waking up other nodes",
    "30013": "ConnectEMC service not configured",
    "30014": "Error occurred when initializing SSL content for remote repository",
    "30015": "No node available to execute operations",
    "30016": "Object is invalid",
    "30018": "Error occurred while releasing lock not by the lock owner",
    "30020": "Invalid software version",
    "30021": "Local repository error occurred",
    "30022": "Remote repository error occurred",
    "30023": "Error occurred while calling internal api",
    "30024": "Internal exception occurred",
    "30025": "Error occurred while calling coordinator client",
    "30026": "Service is busy",
    "30027": "Error occurred while powering off other nodes",
    "30028": "Failed to download a new image",
    "30029": "Error occurred while restarting service",
    "30030": "Failed connecting to controller",
    "40000": "Datastore creation failed",
    "40001": "Datastore deletion failed",
    "40002": "Project is invalid",
    "40003": "Project not found for namespace",
    "40004": "ObjectStore is invalid",
    "40005": "ObjectStore not found for namespace",
    "40006": "ObjectStore being used is not compatible with the request",
    "40007": "ObjectStore does not have associated datastore",
    "40008": "Bucket already exists",
    "40009": "Invalid bucket name",
    "40010": "Bucket Owner not valid",
    "40011": "Invalid object virtual pool type",
    "40012": "Dataservice Invalid Varray",
    "40013": "No Data Store",
    "40014": "System being initialized",
    "40015": "Obj Vpool Lists Not Mutually Exclusive",
    "40016": "Internal error while listing data nodes",
    "50000": "Cannot connect the remote vdc",
    "50001": "Internal error occurred during vdc management operations",
    "50002": "Cannot acquire global lock for vdc operations",
    "50003": "The federation has more than two different versions",
    "50004": "The federation is unstable",
    "51001": "Invalid vdc status occurred when connecting vdc",
    "51002": "Precheck of adding vdc failed",
    "51003": "Failed to generate the new vdc config info",
    "51004": "Sync new cert failed",
    "51005": "Generate cert chain failed",
    "51006": "Sync vdc configuration of connecting vdc failed",
    "51007": "Postcheck of connecting vdc failed",
    "51008": "Update vdc status failed in connecting vdc",
    "52001": "Precheck of removing vdc failed",
    "52002": "Sync vdc configuration of removing vdc failed",
    "52003": "Postcheck of removing vdc failed",
    "52004": "Invalid vdc status occurred when removing vdc",
    "53001": "Cannot update all the vdc",
    "53002": "Invalid vdc status occurred when updating vdc",
    "53003": "Precheck of updating vdc failed",
    "54001": "The current VDC is not in correct status to perform disconnecting operation",
    "54002": "The VDC to be disconnected is still reachable",
    "54003": "Some vdc is under disconnecting or connected failed status",
    "54004": "Failed to disconnect the VDC",
    "55001": "Failed to reconnect the VDC",
    "55002": "The VDC is not in correct status to finish reconnect",
    "55003": "Back end node repair failed when perform reconnect opertion",
    "55004": "There is at least one vdc is unreachable with operator"
}



from ecsclient.common.cas import cas
import logging

log = logging.getLogger(__name__)


class Cas(cas.Cas):

    def update_cluster_id(self, cluster_id):
        """
        Updates the cluster ID

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param cluster_id: given cluster ID
        """
        log.info("Updating cluster ID '{}'".format(cluster_id))
        return self.conn.put('object/user-cas/cluster/{}'.format(cluster_id))



# Standard lib imports
import logging

# Third party imports
# None

# Project level imports
# None


log = logging.getLogger(__name__)


class Licensing(object):
    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_license(self):
        """
        Gets the currently configured licenses.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'license_feature': [
                {
                    u'notice': u'ACTIVATEDTOLicenseSiteNumber:
                                PTA06JUN20131086059',
                    u'trial_license_ind': False,
                    u'expired_ind': False,
                    u'licensed_ind': True,
                    u'site_id': u'UNKNOWN',
                    u'product': u'PXTYD1DZK59Y4C',
                    u'issued_date': u'01/10/2014',
                    u'version': u'2.0',
                    u'storage_capacity': u'TB',
                    u'license_id_indicator': u'U',
                    u'model': u'ViPR_Block',
                    u'serial': u'PRTYD1DZK59X9A',
                    u'issuer': u'EMC'
                },
                {
                    u'notice': u'ACTIVATEDTOLicenseSiteNumber:
                                PTA06JUN20131086059',
                    u'trial_license_ind': False,
                    u'expired_ind': False,
                    u'licensed_ind': True,
                    u'site_id': u'UNKNOWN',
                    u'product': u'PXTYD1DZK59Y4C',
                    u'issued_date': u'01/10/2014',
                    u'version': u'2.0',
                    u'storage_capacity': u'TB',
                    u'license_id_indicator': u'U',
                    u'model': u'ViPR_Commodity',
                    u'serial': u'PRTYD1DZK59X9A',
                    u'issuer': u'EMC'
                }
            ],
            u'license_text': u'LONG-LONG--TEXT'
        }
        """
        log.info("Retrieving license")
        return self.conn.get('license')

    def add_license(self, license_text):
        """
        Adds specified license.

        Required role(s):

        SYSTEM_ADMIN

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param license_text: License text to be added
        """
        payload = {
            "license_text": license_text
        }
        log.info("Adding a new license")
        return self.conn.post('license.json', json_payload=payload)



import time

from ecsclient import schemas
from ecsclient.common.exceptions import ECSClientException
from tests import functional


class TestSecretKey(functional.BaseTestCase):
    def __init__(self, *args, **kwargs):
        super(TestSecretKey, self).__init__(*args, **kwargs)
        self.namespace_1 = "functional-tests-namespace-%s" % int(time.time())
        self.object_user_1 = "functional-tests-objectuser-%s" % int(time.time())
        self.object_user_current = None
        self.secret_key_1 = "E3GooHbvQXSxHplji1aPEGZilIEWYfveJQtFIrAF"
        self.current_user_created = False

    def setUp(self):
        super(TestSecretKey, self).setUp()
        self.client.namespace.create(self.namespace_1)
        self.client.object_user.create(self.object_user_1, self.namespace_1)
        self.client.secret_key.create(user_id=self.object_user_1,
                                      namespace=self.namespace_1,
                                      secret_key=self.secret_key_1)
        r = self.client.user_info.whoami()
        self.object_user_current = r['common_name']
        try:
            self.client.object_user.create(self.object_user_current, self.namespace_1)
            self.current_user_created = True
        except ECSClientException:
            pass
        self.client.secret_key.create()

    def tearDown(self):
        super(TestSecretKey, self).tearDown()
        self.client.object_user.delete(self.object_user_1)
        if self.current_user_created:
            self.client.object_user.delete(self.object_user_current)
        self.client.namespace.delete(self.namespace_1)

    def test_secret_key_get_authenticated_user(self):
        response = self.client.secret_key.get()
        self.assertValidSchema(response, schemas.SECRET_KEYS)

    def test_secret_key_get_by_user(self):
        response = self.client.secret_key.get(user_id=self.object_user_1,
                                              namespace=self.namespace_1)
        self.assertValidSchema(response, schemas.SECRET_KEYS)
        self.assertEqual(response['secret_key_1'], self.secret_key_1)

    def test_secret_key_create_authenticated_user(self):
        response = self.client.secret_key.create()
        self.assertValidSchema(response, schemas.SECRET_KEY)

    def test_secret_key_create_for_user(self):
        secret_key = 'zDR3nIO9ywbdpabxKSbnB3NKGIclBSEpyCkh0KJB'
        response = self.client.secret_key.create(user_id=self.object_user_1,
                                                 namespace=self.namespace_1,
                                                 expiry_time=60,
                                                 secret_key=secret_key)
        self.assertValidSchema(response, schemas.SECRET_KEY)
        self.assertEqual(response['secret_key'], secret_key)

    def test_secret_key_delete_authenticated_user(self):
        response = self.client.secret_key.get()
        self.assertNotEqual(response['secret_key_1'] or response['secret_key_2'], "")

        self.client.secret_key.delete()

        response = self.client.secret_key.get()
        self.assertEqual(response['secret_key_1'], "")
        self.assertEqual(response['secret_key_2'], "")

    def test_secret_key_delete_for_user(self):
        response = self.client.secret_key.get(user_id=self.object_user_1, namespace=self.namespace_1)
        self.assertNotEqual(response['secret_key_1'] or response['secret_key_2'], "")

        self.client.secret_key.delete(user_id=self.object_user_1, namespace=self.namespace_1)

        response = self.client.secret_key.get(user_id=self.object_user_1, namespace=self.namespace_1)
        self.assertEqual(response['secret_key_1'], "")
        self.assertEqual(response['secret_key_2'], "")

    def test_secret_key_replace(self):
        secret_key = 'A3GooHbvQXSxHplji1aPEGZilIEWYfveJQtFIrAF'

        self.client.secret_key.delete(user_id=self.object_user_1, namespace=self.namespace_1)

        response = self.client.secret_key.get(user_id=self.object_user_1, namespace=self.namespace_1)
        self.assertEqual(response['secret_key_1'], "")
        self.assertEqual(response['secret_key_2'], "")

        self.client.secret_key.create(user_id=self.object_user_1,
                                      namespace=self.namespace_1,
                                      secret_key=secret_key)

        response = self.client.secret_key.get(user_id=self.object_user_1, namespace=self.namespace_1)
        self.assertEqual(response['secret_key_1'], secret_key)
        self.assertEqual(response['secret_key_2'], "")



import testtools
from mock import mock
from requests_mock.contrib import fixture
from mock import MagicMock
from mock import mock_open
from mock import patch
from six.moves import http_client
from ecsclient.common.token_request import TokenRequest
from ecsclient.common.exceptions import ECSClientException


class TestTokenRequest(testtools.TestCase):

    def setUp(self):
        super(TestTokenRequest, self).setUp()
        self.token_file_contents = '123TOKEN'
        self.response = MagicMock()
        self.session_get = 'ecsclient.common.token_request.requests.Session.get'

        self.token_request = TokenRequest(username='someone',
                                          password='password',
                                          ecs_endpoint='https://127.0.0.1:4443',
                                          token_endpoint='https://127.0.0.1:4443/login',
                                          verify_ssl=False,
                                          token_path='/tmp/ecstoken.tkn',
                                          request_timeout=5.0,
                                          cache_token=True)
        self.requests_mock = self.useFixture(fixture.Fixture())

    def test_should_get_existing_token(self):
        with patch('os.path.isfile', return_value=True),\
            patch('six.moves.builtins.open', mock_open(read_data='123TOKEN'),
                  create=True):
            self.assertEqual(self.token_request._get_existing_token(),
                             self.token_file_contents)

    def test_should_not_get_existing_token(self):
        with patch('os.path.isfile', return_value=False):
            self.assertEqual(self.token_request._get_existing_token(), None)

    def test_get_new_token_should_throw_ecsclientexception_500(self):
        self.requests_mock.register_uri('GET', 'https://127.0.0.1:4443/login',
                                        status_code=http_client.INTERNAL_SERVER_ERROR)

        with super(testtools.TestCase, self).assertRaises(ECSClientException) as error:
            self.token_request.get_new_token()

        exception = error.exception
        self.assertEqual(exception.http_status, http_client.INTERNAL_SERVER_ERROR)

    def test_get_new_token_should_throw_ecsclientexception_401(self):
        self.requests_mock.register_uri('GET', 'https://127.0.0.1:4443/login',
                                        status_code=http_client.UNAUTHORIZED)

        with super(testtools.TestCase, self).assertRaises(ECSClientException) as error:
            self.token_request.get_new_token()

        exception = error.exception
        self.assertEqual(exception.http_status, http_client.UNAUTHORIZED)

    @mock.patch('ecsclient.common.token_request.os.path.isdir')
    def test_get_new_token_cache_invalid_token_path(self, mock_isdir):
        self.requests_mock.register_uri('GET', 'https://127.0.0.1:4443/login',
                                        headers={'X-SDS-AUTH-TOKEN': 'NEW-TOKEN-123'})
        mock_isdir.side_effect = lambda dir_: dir_ != '/foo/bar'
        self.token_request.cache_token = True
        self.token_request.token_path = '/foo/bar/token.txt'

        with super(testtools.TestCase, self).assertRaises(ECSClientException) as error:
            self.token_request.get_new_token()

        exception = error.exception
        self.assertEqual(exception.message, "Token directory not found")

    @mock.patch('ecsclient.common.token_request.TokenRequest.get_new_token')
    @mock.patch('ecsclient.common.token_request.TokenRequest._get_existing_token')
    def test_token_validation_401(self, mock_get_existing_token, mock_get_new_token):
        self.requests_mock.register_uri('GET', 'https://127.0.0.1:4443/user/whoami',
                                        status_code=http_client.UNAUTHORIZED)
        mock_get_new_token.return_value = 'NEW-TOKEN-123'
        mock_get_existing_token.return_value = 'EXISTING-TOKEN-123'

        token = self.token_request.get_token()

        self.assertEqual(token, 'NEW-TOKEN-123')

    @mock.patch('ecsclient.common.token_request.TokenRequest._get_existing_token')
    def test_token_validation_500(self, mock_get_existing_token):
        self.requests_mock.register_uri('GET', 'https://127.0.0.1:4443/user/whoami',
                                        status_code=http_client.INTERNAL_SERVER_ERROR)
        mock_get_existing_token.return_value = 'EXISTING-TOKEN-123'

        with super(testtools.TestCase, self).assertRaises(ECSClientException) as error:
            self.token_request.get_token()

        exception = error.exception
        self.assertEqual(exception.message, "Token validation error (Code: 500)")
        self.assertEqual(exception.http_status, http_client.INTERNAL_SERVER_ERROR)



#!/usr/bin/env python
try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools

    use_setuptools()
    from setuptools import setup, find_packages


def read(relative):
    """
    Read file contents and return a list of lines.
    ie, read the VERSION file
    """
    contents = open(relative, 'r').read()
    return [l for l in contents.split('\n') if l != '']


with open('README.rst', 'r') as f:
    readme = f.read()

setup(
    name='python-ecsclient',
    url='https://github.com/EMCECS/python-ecsclient',
    keywords=['ecsclient'],
    version=read('VERSION')[0],
    description='A library for interacting with the ECS Management API',
    author='ECS',
    author_email='ecs@dell.com',
    tests_require=read('./test-requirements.txt'),
    install_requires=read('./requirements.txt'),
    test_suite='nose.collector',
    zip_safe=False,
    include_package_data=True,
    packages=find_packages(exclude=['ez_setup']),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



import datetime
import logging
from jsonschema import validate, FormatChecker

log = logging.getLogger(__name__)


def get_formatted_time_string(year, month, day, hour, minute=None):
    """
    Validates input parameters: year, month, day, hour and minute
    and returns the timestamp if valid

    :param year: Four digit year
    :param month: The month
    :param day: The day
    :param hour: The hour
    :param minute: The minutes (optional)
    :return: Returns time stamp in yyyy-MM-dd'T'HH:mm if parameters are valid

    Throws:
        ValueError in case of invalid input
    """

    if minute:
        d = datetime.datetime(int(year), int(month), int(day), int(hour),
                              int(minute))
        return d.strftime("%Y-%m-%dT%H:%M")
    else:
        d = datetime.datetime(int(year), int(month), int(day), int(hour))
        return d.strftime("%Y-%m-%dT%H")


def is_valid_response(response, schema):
    """
    Returns True if the response validates with the schema, False otherwise

    :param response: The response returned by the API
    :param schema: The schema to validate with
    :returns: True if the response validates with the schema, False otherwise
    """
    try:
        validate(response, schema, format_checker=FormatChecker())
        return True
    except Exception as e:
        log.warning("Response is not valid: %s" % (e,))
        return False



WHOAMI = {
    "type": "object",
    "properties": {
        "namespace": {"type": "string"},
        "last_time_password_changed": {
            "type": "string",
            "format": "date-time"
        },
        "distinguished_name": {"type": "string"},
        "common_name": {"type": "string"},
        "roles": {
            "type": "array",
            "items": {
                "type": "string",
                "minLength": 1
            },
            "minItems": 1,
            "uniqueItems": True
        }
    },
    "required": [
        "namespace",
        "last_time_password_changed",
        "distinguished_name",
        "common_name",
        "roles",
    ]
}

LINK = {
    "type": "object",
    "properties": {
        "rel": {"type": "string"},
        "href": {"type": "string"}
    },
    "required": [
        "rel",
        "href"
    ]
}

NAMESPACES = {
    "type": "object",
    "properties": {
        "namespace": {
            "type": "array",
            "minItems": 1,
            "items": {
                "type": "object",
                "properties": {
                    "id": {"type": "string"},
                    "name": {"type": "string"},
                    "link": LINK
                },
                "required": [
                    "id",
                    "name",
                    "link"
                ]
            },
        }
    },
    "required": [
        "namespace"
    ]
}

NAMESPACE = {
    "type": "object",
    "properties": {
        "name": {"type": "string"},
        "id": {"type": "string"},
        "inactive": {"type": "boolean"},
        "is_stale_allowed": {"type": "boolean"},
        "is_encryption_enabled": {"type": ["string", "boolean"]},  # Workaround: The API returns a string instead of
        # a boolean
        "is_compliance_enabled": {"type": "boolean"},
        "global": {"type": ["string", "null"]},
        "remote": {"type": ["string", "null"]},
        "vdc": {"type": ["string", "null"]},
        "default_data_services_vpool": {"type": "string"},
        "allowed_vpools_list": {"type": "array"},
        "disallowed_vpools_list": {"type": "array"},
        "default_bucket_block_size": {"type": "number"},
        "user_mapping": {"type": "array"},
        "link": LINK
    },
    "required": [
        "name",
        "id",
        "inactive",
        "is_stale_allowed",
        "is_encryption_enabled",
        "is_compliance_enabled",
        "global",
        "remote",
        "vdc",
        "default_data_services_vpool",
        "allowed_vpools_list",
        "disallowed_vpools_list",
        "default_bucket_block_size",
        "user_mapping",
        "link"
    ]
}

CERTIFICATE = {
    "type": "object",
    "properties": {
        "chain": {"type": "string"}
    },
    "required": [
        "chain",
    ]
}

LICENSE = {
    "type": "object",
    "properties": {
        "license_feature": {
            "type": "array",
            "minItems": 1,
            "items": {
                "type": "object",
                "properties": {
                    "product": {"type": "string"},
                    "version": {"type": "string"},
                    "model": {"type": "string"},
                    "licensed_ind": {"type": "boolean"},
                    "trial_license_ind": {"type": "boolean"},
                    "storage_capacity": {"type": "string"},
                    "notice": {"type": "string"},
                    "serial": {"type": "string"},
                    "expired_ind": {"type": "boolean"},
                    "issued_date": {"type": "string", "format": "date-time"},
                    "license_id_indicator": {"type": "string"},
                    "issuer": {"type": "string"},
                    "site_id": {"type": "string"}
                },
                "required": [
                    "product",
                    "version",
                    "model",
                    "licensed_ind",
                    "trial_license_ind",
                    "storage_capacity",
                    "notice",
                    "serial",
                    "expired_ind",
                    "issued_date",
                    "license_id_indicator",
                    "issuer",
                    "site_id"
                ]
            },
        },
        "license_text": {"type": "string"}
    },
    "required": [
        "license_feature",
        "license_text"
    ]
}

STORAGE_POOL = {
    "type": "object",
    "properties": {
        "id": {"type": "string"},
        "name": {"type": "string"},
        "isProtected": {"type": "boolean"},
        "isColdStorageEnabled": {"type": "boolean"}
    },
    "required": [
        "id",
        "name",
        "isProtected",
        "isColdStorageEnabled"
    ]
}

STORAGE_POOLS = {
    "type": "object",
    "properties": {
        "varray": {
            "type": "array",
            "minItems": 1,
            "items": STORAGE_POOL
        }
    },
    "required": [
        "varray"
    ]
}

VDC = {
    "type": "object",
    "properties": {
        "id": {"type": "string"},
        "vdcId": {"type": "string"},
        "name": {"type": "string"},
        "vdcName": {"type": "string"},
        "link": LINK,
        "inactive": {"type": "boolean"},
        "local": {"type": "boolean"},
        "global": {"type": ["boolean", "null"]},
        "remote": {"type": ["boolean", "null"]},
        "vdc": {
            "type": ["object", "null"],
            "properties": {
                "id": {"type": "string"},
                "link": {"type": "string"}
            },
            "required": [
                "id",
                "link"
            ]
        },
        "is_encryption_enabled": {"type": "boolean"},
        "managementEndPoints": {"type": "string"},
        "interVdcEndPoints": {"type": "string"},
        "interVdcCmdEndPoints": {"type": "string"},
        "secretKeys": {"type": "string"},
        "permanentlyFailed": {"type": "boolean"}
    },
    "required": [
        "id",
        "vdcId",
        "name",
        "vdcName",
        "link",
        "inactive",
        "local",
        "global",
        "remote",
        "vdc",
        "is_encryption_enabled",
        "managementEndPoints",
        "interVdcEndPoints",
        "interVdcCmdEndPoints",
        "secretKeys",
        "permanentlyFailed"
    ]
}

VDCS = {
    "type": "object",
    "properties": {
        "vdc": {
            "type": "array",
            "minItems": 1,
            "items": VDC
        }
    },
    "required": [
        "vdc"
    ]
}

REPLICATION_GROUP = {
    "type": "object",
    "properties": {
        "id": {"type": "string"},
        "name": {"type": "string"},
        "inactive": {"type": "boolean"},
        "global": {"type": ["boolean", "null"]},
        "remote": {"type": ["boolean", "null"]},
        "vdc": {
            "type": ["object", "null"],
            "properties": {
                "id": {"type": "string"},
                "link": {"type": "string"}
            },
            "required": [
                "id",
                "link"
            ]
        },
        "description": {"type": "string"},
        "varrayMappings": {
            "type": "array",
            "items": {
                "type": "object",
                "properties": {
                    "name": {"type": "string"},
                    "value": {"type": "string"}
                },
                "required": [
                    "name",
                    "value"
                ]
            }
        },
        "creation_time": {"type": "number"},
        "isAllowAllNamespaces": {"type": "boolean"},
        "enable_rebalancing": {"type": "boolean"},
        "isFullRep": {"type": "boolean"}
    },
    "required": [
        "id",
        "name",
        "global",
        "remote",
        "vdc",
        "description",
        "varrayMappings",
        "isAllowAllNamespaces",
        "enable_rebalancing",
        "isFullRep"
    ]
}

REPLICATION_GROUPS = {
    "type": "object",
    "properties": {
        "data_service_vpool": {
            "type": "array",
            "minItems": 1,
            "items": REPLICATION_GROUP
        }
    },
    "required": [
        "data_service_vpool"
    ]
}

DATA_STORES = {
    "type": "object",
    "properties": {
        "data_store": {
            "type": "array",
            "minItems": 1,
            "items": {
                "type": "object",
                "properties": {
                    "id": {"type": "string"},
                    "name": {"type": "string"},
                    "resource_type": {"type": "string"},
                    "link": LINK
                },
                "required": [
                    "id",
                    "name",
                    "resource_type",
                    "link"
                ]
            }
        }
    },
    "required": [
        "data_store"
    ]
}

DATA_STORE = {
    "type": "object",
    "properties": {
        "id": {"type": "string"},
        "name": {"type": "string"},
        "description": {"type": "string"},
        "link": LINK,
        "remote": {"type": ["boolean", "null"]},
        "global": {"type": ["boolean", "null"]},
        "vdc": {"type": ["string", "null"]},
        "creation_time": {"type": "number"},
        "device_info": {"type": "string"},
        "varray": {"type": "string"},
        "device_state": {"type": "string"},
        "usable_gb": {"type": "number"},
        "used_gb": {"type": "number"},
        "free_gb": {"type": "number"}
    },
    "required": [
        "id",
        "name",
        "description",
        "link",
        "remote",
        "global",
        "vdc",
        "creation_time",
        "device_info",
        "varray",
        "device_state",
        "usable_gb",
        "used_gb",
        "free_gb"
    ]
}

DATA_STORES_COMMODITY = {
    "type": "object",
    "properties": {
        "commodity_data_store": {
            "type": "array",
            "minItems": 1,
            "items": DATA_STORE
        }
    },
    "required": [
        "commodity_data_store"
    ]
}

DATA_STORE_TASK = {
    "type": "object",
    "properties": {
        "op_id": {"type": "string"},
        "resource": {
            "type": "object",
            "properties": {
                "link": LINK,
                "id": {"type": "string"},
                "name": {"type": "string"}
            },
            "required": [
                "link",
                "id",
                "name"
            ]
        },
        "name": {"type": "string"},
        "remote": {"type": ["boolean", "null"]},
        "global": {"type": ["boolean", "null"]},
        "vdc": {"type": ["string", "null"]},
        "link": LINK,
        "restLink": {"type": ["object", "null"]},
        "state": {"type": "string"},
        "associated_resources": {"type": "array"}
    },
    "required": [
        "op_id",
        "resource",
        "name",
        "global",
        "remote",
        "vdc",
        "link",
        "restLink",
        "state",
        "associated_resources",
    ]
}

DATA_STORE_TASKS = {
    "type": "object",
    "properties": {
        "task": {
            "type": "array",
            "minItems": 1,
            "items": DATA_STORE_TASK
        }
    },
    "required": [
        "task"
    ]
}

OBJECT_USER = {
    "type": "object",
    "properties": {
        "name": {"type": "string"},
        "created": {
            "type": "string",
            "format": "date-time"
        },
        "namespace": {"type": "string"},
        "locked": {"type": "boolean"}
    },
    "required": [
        "name",
        "created",
        "namespace",
        "locked"
    ]
}

OBJECT_USER_2 = {
    "type": "object",
    "properties": {
        "name": {"type": "string"},
        "namespace": {"type": "string"},
    },
    "required": [
        "userid",
        "namespace"
    ]
}

OBJECT_USERS = {
    "type": "object",
    "properties": {
        "blobuser": {
            "type": "array",
            "minItems": 0,
            "items": OBJECT_USER_2
        },
        "Filter": {"type": "string"}
    },
    "required": [
        "blobuser"
    ]
}

SECRET_KEYS = {
    "type": "object",
    "properties": {
        "secret_key_1": {"type": "string"},
        "secret_key_2": {"type": "string"},
        "key_timestamp_1": {"type": "string"},
        "key_timestamp_2": {"type": "string"},
        "key_expiry_timestamp_1": {"type": "string"},
        "key_expiry_timestamp_2": {"type": "string"},
        "link": LINK
    },
    "required": [
        "secret_key_1",
        "secret_key_2",
        "key_timestamp_1",
        "key_timestamp_2"
    ]
}

SECRET_KEY = {
    "type": "object",
    "properties": {
        "secret_key": {"type": "string"},
        "key_timestamp": {"type": "string"},
        "key_expiry_timestamp": {"type": "string"},
        "link": LINK
    },
    "required": [
        "secret_key",
        "key_timestamp",
        "key_expiry_timestamp",
        "link"
    ]
}

BUCKET_SHORT = {
    "type": "object",
    "properties": {
        "TagSet": {"type": "array"},
        "id": {"type": "string"},
        "global": {"type": ["boolean", "null"]},
        "vdc": {"type": ["string", "null"]},
        "inactive": {"type": "boolean"},
        "metaData": {
            "type": "object",
            "properties": {
                "search_enabled": {"type": "boolean"},
                "max_keys": {"type": "number"},
                "metadata": {"type": "array"}
            },
            "required": [
                "search_enabled",
                "max_keys",
                "metadata"
            ]
        },
        "remote": {"type": ["boolean", "null"]},
        "name": {"type": "string"}
    },
    "required": [
        "TagSet",
        "global",
        "vdc",
        "inactive",
        "metaData",
        "remote",
        "name"
    ]
}

BUCKET = {
    "type": "object",
    "properties": {
        "TagSet": {"type": "array"},
        "id": {"type": "string"},
        "global": {"type": ["boolean", "null"]},
        "vdc": {"type": ["string", "null"]},
        "search_metadata": {
            "type": "object",
            "properties": {
                "search_enabled": {"type": "boolean"},
                "max_keys": {"type": "number"},
                "metadata": {"type": "array"}
            },
            "required": [
                "search_enabled",
                "max_keys",
                "metadata"
            ]
        },
        "remote": {"type": ["boolean", "null"]},
        "name": {"type": "string"},
        "block_size": {"type": "number"},
        "retention": {"type": "number"},
        "api_type": {"type": "string"},
        "notification_size": {"type": "number"},
        "namespace": {"type": "string"},
        "default_retention": {"type": "number"},
        "locked": {"type": "boolean"},
        "is_encryption_enabled": {"type": ["boolean", "string"]},
        "is_stale_allowed": {"type": "boolean"},
        "vpool": {"type": "string"},
        "fs_access_enabled": {"type": "boolean"},
        "created": {"type": "string", "format": "date-time"},
        "owner": {"type": "string"}
    },
    "required": [
        "TagSet",
        "global",
        "vdc",
        "search_metadata",
        "remote",
        "name",
        "block_size",
        "retention",
        "api_type",
        "notification_size",
        "namespace",
        "default_retention",
        "locked",
        "is_encryption_enabled",
        "is_stale_allowed",
        "vpool",
        "fs_access_enabled",
        "created",
        "owner"
    ]
}

BUCKET_LIST = {
    "type": "object",
    "properties": {
        "object_bucket": {
            "type": "array",
            "minItems": 0,
            "items": BUCKET
        },
        "Filter": {"type": "string"},
        "MaxBuckets": {"type": "number"}
    },
    "required": [
        "object_bucket",
        "Filter",
        "MaxBuckets"
    ]
}

BUCKET_ACL = {
    "type": "array",
    "minItems": 1,
    "items": {
        "type": "object",
        "properties": {
            "display_name": {"type": "string"},
            "description": {"type": "string"},
            "id": {"type": "string"}
        },
        "required": [
            "display_name",
            "id"
        ]
    }
}

BUCKET_ACL_PERMISSIONS = {
    "type": "object",
    "properties": {
        "permission": BUCKET_ACL
    },
    "required": [
        "permission"
    ]
}

BUCKET_ACL_GROUPS = {
    "type": "object",
    "properties": {
        "group": BUCKET_ACL
    },
    "required": [
        "group"
    ]
}

BUCKET_USER_METADATA = {
    "type": "object",
    "properties": {
        "metadata": {
            "type": "array",
            "minItems": 1,
            "items": {
                "type": "object",
                "properties": {
                    "name": {"type": "string"},
                    "value": {"type": "string"}
                },
                "required": [
                    "name",
                    "value"
                ]
            }
        },
        "head_type": {"type": "string"}
    },
    "required": [
        "metadata",
        "head_type"
    ]
}

BUCKET_SYSTEM_METADATA = {
    "type": "object",
    "properties": {
        "metadata": {
            "type": "array",
            "minItems": 1,
            "items": {
                "type": "object",
                "properties": {
                    "name": {"type": "string"},
                    "datatype": {"type": "string"},
                    "type": {"type": "string"}
                },
                "required": [
                    "name",
                    "datatype",
                    "type"
                ]
            }
        },
        "max_keys": {"type": "number"}
    },
    "required": [
        "metadata",
        "max_keys"
    ]
}

MANAGEMENT_USER = {
    "type": "object",
    "properties": {
        "userId": {"type": "string"},
        "isSystemAdmin": {"type": "boolean"},
        "isSystemMonitor": {"type": "boolean"},
        "is_external_group": {"type": "boolean"}
    },
    "required": [
        "userId",
        "isSystemAdmin",
        "isSystemMonitor",
        "is_external_group"
    ]
}

MANAGEMENT_USERS = {
    "type": "object",
    "properties": {
        "mgmt_user_info": {
            "type": "array",
            "minItems": 0,
            "items": MANAGEMENT_USER
        },
    },
    "required": [
        "mgmt_user_info"
    ]
}

GROUP_LIST = {
    "type": "object",
    "properties": {
        "groups_list": {
            "type": "array",
            "items": {
                "type": "string",
                "minLength": 1
            },
            "minItems": 1,
            "uniqueItems": True
        }
    },
    "required": [
        "groups_list"
    ]
}

VDC_KEYSTORE = {
    "type": "object",
    "properties": {
        "chain": {"type": "string"}
    },
    "required": [
        "chain"
    ]
}

NODE = {
    "type": "object",
    "properties": {
        "rackId": {"type": "string"},
        "version": {"type": "string"},
        "nodeid": {"type": "string"},
        "isLocal": {"type": "boolean"},
        "nodename": {"type": "string"},
        "ip": {"type": "string"},
    },
    "required": [
        "rackId",
        "version",
        "nodeid",
        "isLocal",
        "nodename",
        "ip"
    ]
}

NODE_LIST = {
    "type": "object",
    "properties": {
        "node": {
            "type": "array",
            "items": NODE,
            "minItems": 1,
            "uniqueItems": True
        }
    },
    "required": [
        "node"
    ]
}

ALERTS = {
    "type": "object",
    "properties": {
        "MaxAlerts": {"type": "number"},
        "Filter": {"type": "string"},
        "alert": {"type": "array"}
    },
    "required": [
        "MaxAlerts",
        "Filter",
        "alert"
    ]
}

CAPACITY = {
    "type": "object",
    "properties": {
        "totalFree_gb": {"type": "number"},
        "totalProvisioned_gb": {"type": "number"}
    },
    "required": [
        "totalFree_gb",
        "totalProvisioned_gb"
    ]
}



import logging

log = logging.getLogger(__name__)


class Syslog(object):
    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_syslog_servers(self):
        """
        Get list of Syslog Server info.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:
        """
        # TODO: Add example JSON response
        log.info("Fetching syslog servers")
        return self.conn.get('vdc/syslog/config')

    def get_syslog_server(self, syslog_id):
        """
        Get Syslog Server info.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:
        {}

        :param syslog_id: Syslog server ID
        """
        # TODO: Add example JSON response
        log.info("Getting syslog server information with ID '{}'".format(syslog_id))
        return self.conn.get('vdc/syslog/config/{}'.format(syslog_id))

    def create_syslog_server(self, server, port, protocol, severity):
        """
        Creates a Syslog server.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:
        {}

        :param server: Fully qualified domain name or IP
        :param port: Syslog port
        :param protocol: Protocol Syslog protocol UDP/TCP
        :param severity: Severity - minimal syslog message severity for this server
        """
        # TODO: Add example JSON response

        payload = {
          "server": server,
          "port": port,
          "protocol": protocol,
          "severity": severity
        }

        log.info("Creating syslog server")
        return self.conn.post('vdc/syslog/config', json_payload=payload)

    def update_syslog_server(self, syslog_id, server, port, protocol, severity):
        """
        Update specified Syslog Server.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param syslog_id: Syslog server ID
        :param server: Fully qualified domain name or IP
        :param port: Syslog port
        :param protocol: Protocol Syslog protocol UDP/TCP
        :param severity: Severity - minimal syslog message severity for this server
        """

        payload = {
            "server": server,
            "port": port,
            "protocol": protocol,
            "severity": severity
        }
        log.info("Updating syslog server with ID '{}'".format(syslog_id))
        return self.conn.put('vdc/syslog/config/{}'.format(syslog_id), json_payload=payload)

    def delete_syslog_server(self, syslog_id):
        """
        Delete specified Syslog Server.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param syslog_id: Syslog server ID
        """
        log.info("Deleting syslog server with ID '{}'".format(syslog_id))
        return self.conn.delete('vdc/syslog/config/{}'.format(syslog_id))



from ecsclient.common.user_management import authentication_provider, secret_key, \
    password_group, management_user, object_user

authentication_provider = authentication_provider
secret_key = secret_key
password_group = password_group
management_user = management_user
object_user = object_user



import logging

log = logging.getLogger(__name__)


class VirtualDataCenter(object):
    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def list(self):
        """
        Gets all details of all configured VDCs.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        SECURITY_ADMIN

        Example JSON result from the API:

        {
            u'vdc': [
                {
                    u'remote': None,
                    u'name': u'tiva01',
                    u'tags': [

                    ],
                    u'global': None,
                    u'interVdcEndPoints': u'192.29.3.48,
                    172.29.3.149,
                    172.29.3.150,
                    172.29.3.151,
                    172.29.3.152,
                    172.29.3.153,
                    172.29.3.154,
                    172.29.3.155',
                    u'vdcId': u'urn: storageos: VirtualDataCenterData:
                                a9faea85-d377-4a42-b5f1-fa15829f0c33',
                    u'vdc': None,
                    u'inactive': False,
                    u'link': {
                        u'href': u'/object/vdcs/vdc/tiva01',
                        u'rel': u'self'
                    },
                    u'secretKeys': u'55fmIFBniRuCBVx327Av',
                    u'vdcName': u'tiva01',
                    u'local': True,
                    u'id': u'urn: storageos: VirtualDataCenterData:
                                a9faea85-d377-4a42-b5f1-fa15829f0c33',
                    u'permanentlyFailed': False
                },
                {
                    u'remote': None,
                    u'name': u'tiva02',
                    u'tags': [

                    ],
                    u'global': None,
                    u'interVdcEndPoints': u'192.29.3.12,
                    172.29.3.213,
                    172.29.3.214,
                    172.29.3.215,
                    172.29.3.216,
                    172.29.3.217,
                    172.29.3.218,
                    172.29.3.219',
                    u'vdcId': u'urn: storageos: VirtualDataCenterData:
                                7d7d2ed5-e253-4be9-b8bb-5ff5b2697e6f',
                    u'vdc': None,
                    u'inactive': False,
                    u'link': {
                        u'href': u'/object/vdcs/vdc/tiva02',
                        u'rel': u'self'
                    },
                    u'secretKeys': u'99TilKh9aTwt9fLbCiKd',
                    u'vdcName': u'tiva02',
                    u'local': False,
                    u'id': u'urn: storageos: VirtualDataCenterData:
                                7d7d2ed5-e253-4be9-b8bb-5ff5b2697e6f',
                    u'permanentlyFailed': False
                }
            ]
        }
        """
        log.info('Listing VDCs')
        return self.conn.get('object/vdcs/vdc/list')

    def get(self, vdc_id=None, name=None):
        """
        Gets the details for a VDC by its ID or its name. Either of the two have to be set.
        If both are set, only the ID is used.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'remote': None,
            u'name': u'tiva01',
            u'tags': [

            ],
            u'global': None,
            u'interVdcEndPoints': u'192.29.3.48,
            172.29.3.149,
            172.29.3.150,
            172.29.3.151,
            172.29.3.152,
            172.29.3.153,
            172.29.3.154,
            172.29.3.155',
            u'vdcId': u'urn: storageos: VirtualDataCenterData:
                        a9faea85-d377-4a42-b5f1-fa15829f0c33',
            u'vdc': None,
            u'inactive': False,
            u'link': {
                u'href': u'/object/vdcs/vdc/tiva01',
                u'rel': u'self'
            },
            u'secretKeys': u'55fmIFBniRuCBVx327Av',
            u'vdcName': u'tiva01',
            u'local': True,
            u'id': u'urn: storageos: VirtualDataCenterData:
                        a9faea85-d377-4a42-b5f1-fa15829f0c33',
            u'permanentlyFailed': False
        }

        param: vdc_id: VDC identifier for which VDC Information is to be retrieved.
        param: name: VDC name for which VDC Information is to be retrieved.
        """
        if not vdc_id and not name:
            raise ValueError('Either the ID or the name have to be set.')

        if vdc_id:
            log.info("Getting VDC by ID '{}'".format(vdc_id))
            url = 'object/vdcs/vdcid/{}'.format(vdc_id)
        else:
            log.info("Getting VDC by name '{}'".format(name))
            url = 'object/vdcs/vdc/{}'.format(name)

        return self.conn.get(url)

    def get_local(self):
        """
        Gets the details for the local VDC.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        SECURITY_ADMIN

        Example JSON result from the API:

        {
            u'remote': None,
            u'name': u'tiva01',
            u'tags': [

            ],
            u'global': None,
            u'interVdcEndPoints': u'192.29.3.48,
            172.29.3.149,
            172.29.3.150,
            172.29.3.151,
            172.29.3.152,
            172.29.3.153,
            172.29.3.154,
            172.29.3.155',
            u'vdcId': u'urn: storageos: VirtualDataCenterData:
                        a9faea85-d377-4a42-b5f1-fa15829f0c33',
            u'vdc': None,
            u'inactive': False,
            u'link': {
                u'href': u'/object/vdcs/vdc/tiva01',
                u'rel': u'self'
            },
            u'secretKeys': u'55fmIFBniRuCBVx327Av',
            u'vdcName': u'tiva01',
            u'local': True,
            u'id': u'urn: storageos: VirtualDataCenterData:
                        a9faea85-d377-4a42-b5f1-fa15829f0c33',
            u'permanentlyFailed': False
        }
        """
        return self.get(name='local')

    def get_local_secret_key(self):
        """
        Gets the secret key for the local VDC.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {u'key': u'55fmIFBniRuCBVx327Av'}
        """
        log.info('Getting local VDC secret key')
        return self.conn.get('object/vdcs/vdc/local/secretkey')

    def update(self, vdc_name, inter_vdc_endpoints, secret_key, new_name=None,
               inter_vdc_cmd_endpoints=None, management_endpoints=None):
        """
        Update the attributes for the current VDC or a VDC which you want the
        current VDC to connect. Enables the name of the VDC, the end points
        that can be used to communicate with it, and a secret key used to
        encrypt traffic between VDCs to be set.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param vdc_name: VDC name for which mapping needs to be update
        :param new_name: Name of VDC to be updated (optional)
        :param inter_vdc_endpoints: Endpoints for the VDC
        :param secret_key: Secret key to encrypt communication between VDC
        :param inter_vdc_cmd_endpoints: Control Plane endpoints for the VDC (optional)
        :param management_endpoints: The management end points for the VDC (optional)
        """
        if not new_name:
            new_name = vdc_name

        payload = {
            'vdcName': new_name,
            'interVdcEndPoints': inter_vdc_endpoints,
            'secretKeys': secret_key
        }
        if inter_vdc_cmd_endpoints:
            payload['interVdcCmdEndPoints'] = inter_vdc_cmd_endpoints
        if management_endpoints:
            payload['managementEndPoints'] = management_endpoints

        log.info("Updating the VDC with name '{}'".format(vdc_name))
        return self.conn.put('object/vdcs/vdc/{}'.format(vdc_name), json_payload=payload)

    def delete(self, vdc_id):
        """
        Deactivates and deletes a VDC. Enables attributes for the current VDC
        to be deleted and enables information held by a VDC about other VDCs
        to be deleted.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param vdc_id: VDC identifier for which VDC Information needs to be
        deleted
        """
        log.info("Deleting VDC '{}'".format(vdc_id))

        return self.conn.post('object/vdcs/vdc/{}/deactivate'.format(vdc_id))



import time

from ecsclient import schemas
from ecsclient.common.exceptions import ECSClientException
from tests import functional


class TestBucket(functional.BaseTestCase):
    def __init__(self, *args, **kwargs):
        super(TestBucket, self).__init__(*args, **kwargs)
        self.bucket_1 = "functional-tests-bucket-%s" % int(time.time())
        self.bucket_2 = self.bucket_1 + '_second'
        self.namespace_1 = "functional-tests-namespace-%s" % int(time.time())
        self.replication_group_1 = "functional-tests-replicationgroup-%s" % int(time.time())
        self.replication_group_1_id = "placeholder"
        self.storage_pool_1 = "functional-tests-storagepool-%s" % int(time.time())
        self.object_user = None
        self.object_user_created = False
        self.replication_group_created = False

    def setUp(self):
        super(TestBucket, self).setUp()
        # Create a namespace
        self.client.namespace.create(self.namespace_1)
        # Create an object user with the current logged in user name
        r = self.client.user_info.whoami()
        self.object_user = r['common_name']
        try:
            self.client.object_user.create(self.object_user, self.namespace_1)
            self.object_user_created = True
        except ECSClientException:
            pass
        # Create a Replication Group
        r = self.client.replication_group.list()
        if len(r['data_service_vpool']) > 0:
            self.replication_group_created = False
            self.replication_group_1_id = r['data_service_vpool'][0]['id']
        else:
            r = self.client.vdc.get_local()
            self.vdc_1_id = r['id']
            r = self.client.storage_pool.create(self.storage_pool_1)
            self.storage_pool_1_id = r['id']
            zone_mappings = [(
                self.vdc_1_id,
                self.storage_pool_1_id
            )]
            r = self.client.replication_group.create(self.replication_group_1, zone_mappings)
            self.replication_group_created = True
            self.replication_group_1_id = r['id']

        self.client.bucket.create(self.bucket_1,
                                  namespace=self.namespace_1,
                                  replication_group=self.replication_group_1_id)

    def tearDown(self):
        super(TestBucket, self).tearDown()
        for bucket in [self.bucket_1,
                       self.bucket_2]:
            try:
                self.client.bucket.delete(bucket, namespace=self.namespace_1)
            except ECSClientException:
                pass

        if self.object_user_created:
            self.client.object_user.delete(self.object_user)
        self.client.namespace.delete(self.namespace_1)
        if self.replication_group_created:
            self.client.storage_pool.delete(self.storage_pool_1_id)
            self.client.replication_group.delete(self.replication_group_1_id)

    def test_bucket_list(self):
        response = self.client.bucket.list(self.namespace_1)
        self.assertValidSchema(response, schemas.BUCKET_LIST)

    def test_bucket_create(self):
        response = self.client.bucket.create(self.bucket_2,
                                             namespace=self.namespace_1,
                                             replication_group=self.replication_group_1_id,
                                             filesystem_enabled=False,
                                             head_type='s3',
                                             stale_allowed=True,
                                             encryption_enabled=False
                                             )
        self.assertValidSchema(response, schemas.BUCKET_SHORT)
        self.assertEqual(response['name'], self.bucket_2)

    def test_bucket_get(self):
        response = self.client.bucket.get(self.bucket_1, namespace=self.namespace_1)
        self.assertValidSchema(response, schemas.BUCKET)
        self.assertEqual(response['name'], self.bucket_1)

    def test_bucket_set_owner(self):
        self.client.bucket.set_owner(self.bucket_1,
                                     self.object_user,
                                     namespace=self.namespace_1)

        response = self.client.bucket.get(self.bucket_1, namespace=self.namespace_1)
        self.assertEqual(response['owner'], self.object_user)

    def test_bucket_set_stale_allowed(self):
        response = self.client.bucket.get(self.bucket_1, namespace=self.namespace_1)
        self.assertEqual(response['is_stale_allowed'], False)

        self.client.bucket.set_stale_allowed(self.bucket_1, True, namespace=self.namespace_1)

        response = self.client.bucket.get(self.bucket_1, namespace=self.namespace_1)
        self.assertEqual(response['is_stale_allowed'], True)

    def test_bucket_lock(self):
        self.client.bucket.set_lock(self.bucket_1, is_locked=True, namespace=self.namespace_1)
        response = self.client.bucket.get_lock(self.bucket_1, namespace=self.namespace_1)
        self.assertEqual(response['isLocked'], True)

        self.client.bucket.set_lock(self.bucket_1, is_locked=False, namespace=self.namespace_1)
        response = self.client.bucket.get_lock(self.bucket_1, namespace=self.namespace_1)
        self.assertEqual(response['isLocked'], False)

    def test_bucket_quota(self):
        self.client.bucket.set_quota(self.bucket_1, namespace=self.namespace_1,
                                     block_size=10,
                                     notification_size=1)
        response = self.client.bucket.get_quota(self.bucket_1, namespace=self.namespace_1)
        self.assertEqual(response['blockSize'], 10)
        self.assertEqual(response['notificationSize'], 1)

        self.client.bucket.delete_quota(self.bucket_1, namespace=self.namespace_1)
        response = self.client.bucket.get_quota(self.bucket_1, namespace=self.namespace_1)
        self.assertEqual(response['blockSize'], -1)
        self.assertEqual(response['notificationSize'], -1)

    def test_bucket_acl_permissions(self):
        response = self.client.bucket.get_acl_permissions()
        self.assertValidSchema(response, schemas.BUCKET_ACL_PERMISSIONS)

    def test_bucket_acl_groups(self):
        response = self.client.bucket.get_acl_groups()
        self.assertValidSchema(response, schemas.BUCKET_ACL_GROUPS)

    def test_bucket_acl(self):
        user_acl = [{'permission': ['full_control'], 'user': self.object_user}]
        group_acl = [{'permission': ['read'], 'group': 'public'}]
        customgroup_acl = [{'permission': ['delete', 'read', 'write'], 'customgroup': 'cgroup1'}]

        self.client.bucket.set_acl(self.bucket_1,
                                   namespace=self.namespace_1,
                                   owner=self.object_user,
                                   default_group='public',
                                   user_acl=user_acl,
                                   group_acl=group_acl,
                                   customgroup_acl=customgroup_acl)

        acl = self.client.bucket.get_acl(self.bucket_1,
                                         namespace=self.namespace_1)

        self.assertEqual(acl['acl']['owner'], self.object_user)
        self.assertEqual(acl['acl']['default_group'], 'public')
        self.assertEqual(acl['acl']['group_acl'], group_acl)
        self.assertEqual(acl['acl']['user_acl'], user_acl)
        acl['acl']['customgroup_acl'][0]['permission'].sort()
        self.assertEqual(acl['acl']['customgroup_acl'], customgroup_acl)
        self.assertEqual(acl['bucket'], self.bucket_1)
        self.assertEqual(acl['namespace'], self.namespace_1)

        # Clear ACLs

        self.client.bucket.set_acl(self.bucket_1,
                                   namespace=self.namespace_1)
        acl = self.client.bucket.get_acl(self.bucket_1,
                                         namespace=self.namespace_1)

        self.assertEqual(acl['acl']['group_acl'], [])
        self.assertEqual(acl['acl']['user_acl'], [])
        self.assertEqual(acl['acl']['customgroup_acl'], [])
        self.assertEqual(acl['bucket'], self.bucket_1)
        self.assertEqual(acl['namespace'], self.namespace_1)

    def test_bucket_user_metadata(self):
        self.client.bucket.set_metadata(self.bucket_1,
                                        "key1",
                                        "value1",
                                        "S3",
                                        namespace=self.namespace_1)

        response = self.client.bucket.get_metadata(self.bucket_1, "S3", namespace=self.namespace_1)
        self.assertValidSchema(response, schemas.BUCKET_USER_METADATA)
        self.assertEqual(response['metadata'][0]['name'], "key1")
        self.assertEqual(response['metadata'][0]['value'], "value1")
        self.assertEqual(response['head_type'], "S3")

        self.client.bucket.delete_metadata(self.bucket_1, "S3", namespace=self.namespace_1)
        f = self.client.bucket.get_metadata
        self.assertRaises(ECSClientException, f, self.bucket_1, "S3", namespace=self.namespace_1)

    def test_bucket_system_metadata(self):
        response = self.client.bucket.get_system_metadata_keys()
        self.assertValidSchema(response, schemas.BUCKET_SYSTEM_METADATA)
